#!/bin/bash
# Script that converts HDF5 SDRs into the requisite HDF4 products for VFIRE375 2.4.2
# Usage:
#	convert_SDRs.sh svi01 <SVI01> svi02 <SVI02> svi03 <SVI03> svi04 <SVI04> svi05 <SVI05> gitco <GITCO> svm13 <SVM13> ivcdb <IVCDB> gmtco <GMTCO> outdir <Output Directory>

# First, find out where we are
case $0 in
         /*)  SHELLFILE=$0 ;;
		./*)  SHELLFILE=${PWD}${0#.} ;;
        ../*) SHELLFILE=${PWD%/*}${0#..} ;;
          *)  SHELLFILE=$(type -P "$0") ; if [ "${SHELLFILE:0:1}" != "/" ]; then SHELLFILE="${PWD}/$SHELLFILE" ; fi ;;
esac

# readlink -m makes sure SHELLDIR is a simple absolute path
SHELLDIR=`readlink -m "${SHELLFILE%/*}"`

# So VIIRS pre-processors are one directory above:
PPDIR="$SHELLDIR/../VIIRS_PREPROCESS/PREPROCESS"

# And ancillary granulator as well:
AGDIR="$SHELLDIR/../anc_granulator/scripts"

# Assign and check arguments:
SVI01=""
SVI02=""
SVI03=""
SVI04=""
SVI05=""
GITCO=""
SVM13=""
IVCDB=""
GMTCO=""
OUTDIR=""

# Parse the input arguments
while [[ "$1" != "" ]]; do
	key="$1"
	shift
	value="$1"
	shift
	case "$key" in
		"svi01") SVI01="$value";;
		"svi02") SVI02="$value";;
		"svi03") SVI03="$value";;
		"svi04") SVI04="$value";;
		"svi05") SVI05="$value";;
		"gitco") GITCO="$value";;
		"svm13") SVM13="$value";;
		"ivcdb") IVCDB="$value";;
		"gmtco") GMTCO="$value";;
		"outdir") OUTDIR="$value";;
	esac
done

# We have to name the HDF4 files so they have the correct timestamp in their filenames...
# Filename format example: NPP_VIAES_L1.Ayyyyddd.hhmm.001.yyyydddhhmmss.hdf

#######################################################################
# Assemble Start Date and Time
#######################################################################
# Parse date info from GITCO HDF5 attributes, and not filename
REFTIMESTAMP1=`$SHELLDIR/h5dump -a /Data_Products/VIIRS-IMG-GEO-TC/VIIRS-IMG-GEO-TC_Aggr/AggregateBeginningDate $GITCO | grep "(0,0)" | sed 's/ //g'`

# Get the year: position 7, size 4
REFYEAR=${REFTIMESTAMP1:7:4}

# Get the year (short): position 9, size 2
REFYEARSHORT=${REFTIMESTAMP1:9:2}

# Get the month: position 11, size 2
REFMONTH=${REFTIMESTAMP1:11:2}

# Get the day of month: position 13, size 2
REFDAY=${REFTIMESTAMP1:13:2}

# Parse time info from GITCO HDF5 attributes, and not filename
REFTIMESTAMP2=`$SHELLDIR/h5dump -a /Data_Products/VIIRS-IMG-GEO-TC/VIIRS-IMG-GEO-TC_Aggr/AggregateBeginningTime $GITCO | grep "(0,0)" | sed 's/ //g'`

# Get the hour: position 7, size 2
REFHOUR=${REFTIMESTAMP2:7:2}

# Get the minutes: position 9, size 2
REFMINS=${REFTIMESTAMP2:9:2}

# Get the seconds: position 11, size 2
REFSECS=${REFTIMESTAMP2:11:2}

# 2.5.1: Get the microseconds: position 14, size 3
REFMICROS=${REFTIMESTAMP2:14:3}

# Get the julian day
REFJUL=`date -d "$REFYEAR-$REFMONTH-$REFDAY $REFHOUR:$REFMINS:$REFSECS" +%j`

# Now assemble the date string
H4DATESTRING="A$REFYEAR$REFJUL.$REFHOUR$REFMINS.001.$REFYEAR$REFJUL$REFHOUR$REFMINS$REFSECS"

# 2.5.1: Also assemble the date/time string for the attribute
STARTDATESTRING="$REFYEAR-$REFMONTH-$REFDAY $REFHOUR:$REFMINS:$REFSECS.$REFMICROS"

#######################################################################
# 2.5.1: Assemble End Date and Time
#######################################################################
# Parse date info from GITCO HDF5 attributes, and not filename
ENDTIMESTAMP1=`$SHELLDIR/h5dump -a /Data_Products/VIIRS-IMG-GEO-TC/VIIRS-IMG-GEO-TC_Aggr/AggregateEndingDate $GITCO | grep "(0,0)" | sed 's/ //g'`

# Get the year: position 7, size 4
ENDYEAR=${ENDTIMESTAMP1:7:4}

# Get the year (short): position 9, size 2
ENDYEARSHORT=${ENDTIMESTAMP1:9:2}

# Get the month: position 11, size 2
ENDMONTH=${ENDTIMESTAMP1:11:2}

# Get the day of month: position 13, size 2
ENDDAY=${ENDTIMESTAMP1:13:2}

# Parse time info from GITCO HDF5 attributes, and not filename
ENDTIMESTAMP2=`$SHELLDIR/h5dump -a /Data_Products/VIIRS-IMG-GEO-TC/VIIRS-IMG-GEO-TC_Aggr/AggregateEndingTime $GITCO | grep "(0,0)" | sed 's/ //g'`

# Get the hour: position 7, size 2
ENDHOUR=${ENDTIMESTAMP2:7:2}

# Get the minutes: position 9, size 2
ENDMINS=${ENDTIMESTAMP2:9:2}

# Get the seconds: position 11, size 2
ENDSECS=${ENDTIMESTAMP2:11:2}

# 2.5.1: Get the microseconds: position 14, size 3
ENDMICROS=${ENDTIMESTAMP2:14:3}

# Get the julian day
ENDJUL=`date -d "$ENDYEAR-$ENDMONTH-$ENDDAY $ENDHOUR:$ENDMINS:$ENDSECS" +%j`

# 2.5.1: Also assemble the date/time string for the attribute
ENDDATESTRING="$ENDYEAR-$ENDMONTH-$ENDDAY $ENDHOUR:$ENDMINS:$ENDSECS.$ENDMICROS"

# Create VIAES product from SVI01 to SVI05 SDRs
eval "$PPDIR/h5SDS_transfer_rename-drl $SVI01 $OUTDIR/NPP_VIAES_L1.$H4DATESTRING.hdf Reflectance Reflectance_I1"
eval "$PPDIR/h5SDS_transfer_rename-drl $SVI02 $OUTDIR/NPP_VIAES_L1.$H4DATESTRING.hdf Reflectance Reflectance_I2"
eval "$PPDIR/h5SDS_transfer_rename-drl $SVI03 $OUTDIR/NPP_VIAES_L1.$H4DATESTRING.hdf Reflectance Reflectance_I3"
eval "$PPDIR/h5SDS_transfer_rename-drl $SVI04 $OUTDIR/NPP_VIAES_L1.$H4DATESTRING.hdf BrightnessTemperature BrightnessTemperature_I4"
eval "$PPDIR/h5SDS_transfer_rename-drl $SVI05 $OUTDIR/NPP_VIAES_L1.$H4DATESTRING.hdf BrightnessTemperature BrightnessTemperature_I5"

eval "$PPDIR/h5SDS_transfer_rename-drl $SVI01 $OUTDIR/NPP_VIAES_L1.$H4DATESTRING.hdf QF1_VIIRSIBANDSDR QF1_VIIRSIBANDSDR_I1"
eval "$PPDIR/h5SDS_transfer_rename-drl $SVI02 $OUTDIR/NPP_VIAES_L1.$H4DATESTRING.hdf QF1_VIIRSIBANDSDR QF1_VIIRSIBANDSDR_I2"
eval "$PPDIR/h5SDS_transfer_rename-drl $SVI03 $OUTDIR/NPP_VIAES_L1.$H4DATESTRING.hdf QF1_VIIRSIBANDSDR QF1_VIIRSIBANDSDR_I3"
eval "$PPDIR/h5SDS_transfer_rename-drl $SVI04 $OUTDIR/NPP_VIAES_L1.$H4DATESTRING.hdf QF1_VIIRSIBANDSDR QF1_VIIRSIBANDSDR_I4"
eval "$PPDIR/h5SDS_transfer_rename-drl $SVI05 $OUTDIR/NPP_VIAES_L1.$H4DATESTRING.hdf QF1_VIIRSIBANDSDR QF1_VIIRSIBANDSDR_I5"

# Create IMFTS product from GITCO geolocation
eval "$PPDIR/h5SDS_transfer_rename-drl $GITCO $OUTDIR/NPP_IMFTS_L1.$H4DATESTRING.hdf Latitude Latitude"
eval "$PPDIR/h5SDS_transfer_rename-drl $GITCO $OUTDIR/NPP_IMFTS_L1.$H4DATESTRING.hdf Longitude Longitude"
eval "$PPDIR/h5SDS_transfer_rename-drl $GITCO $OUTDIR/NPP_IMFTS_L1.$H4DATESTRING.hdf MidTime MidTime"
eval "$PPDIR/h5SDS_transfer_rename-drl $GITCO $OUTDIR/NPP_IMFTS_L1.$H4DATESTRING.hdf SolarZenithAngle SolarZenithAngle"
eval "$PPDIR/h5SDS_transfer_rename-drl $GITCO $OUTDIR/NPP_IMFTS_L1.$H4DATESTRING.hdf SolarAzimuthAngle SolarAzimuthAngle"
eval "$PPDIR/h5SDS_transfer_rename-drl $GITCO $OUTDIR/NPP_IMFTS_L1.$H4DATESTRING.hdf SatelliteZenithAngle SatelliteZenithAngle"
eval "$PPDIR/h5SDS_transfer_rename-drl $GITCO $OUTDIR/NPP_IMFTS_L1.$H4DATESTRING.hdf SatelliteAzimuthAngle SatelliteAzimuthAngle"
eval "$PPDIR/h5SDS_transfer_rename-drl $GITCO $OUTDIR/NPP_IMFTS_L1.$H4DATESTRING.hdf QF2_VIIRSSDRGEO QF2_VIIRSSDRGEO"

# Create VMAES product from SVM13 SDR
eval "$PPDIR/h5SDS_transfer_rename-drl $SVM13 $OUTDIR/NPP_VMAES_L1.$H4DATESTRING.hdf Radiance Radiance_M13"
eval "$PPDIR/h5SDS_transfer_rename-drl $SVM13 $OUTDIR/NPP_VMAES_L1.$H4DATESTRING.hdf QF1_VIIRSMBANDSDR QF1_VIIRSMBANDSDR_M13"

# Create VCDGIPS product from IVCDB unaggregated product
eval "$PPDIR/h5SDS_transfer_rename-drl $IVCDB $OUTDIR/NPP_VCDGIPS_L1.$H4DATESTRING.hdf radiance_6 radiance_6"

# Create QSLWMIP product from GMTCO geolocation
eval "$AGDIR/get-qslwm_vaf $GMTCO true $OUTDIR/NPP_QSLWMIP_L1.$H4DATESTRING.hdf"

# Transfer required HDF5 attributes to IMFTS HDF4
eval "$PPDIR/hdf5_to_hdf4_attr $GITCO $OUTDIR/NPP_IMFTS_L1.$H4DATESTRING.hdf /Data_Products/VIIRS-IMG-GEO-TC/VIIRS-IMG-GEO-TC_Aggr AggregateBeginningDate BeginningDate"
eval "$PPDIR/hdf5_to_hdf4_attr $GITCO $OUTDIR/NPP_IMFTS_L1.$H4DATESTRING.hdf /Data_Products/VIIRS-IMG-GEO-TC/VIIRS-IMG-GEO-TC_Aggr AggregateBeginningTime BeginningTime"

# 2.5.1: Create "StartTime" and "EndTime" attributes for VIAES HDF4
eval "$PPDIR/hdf5_to_hdf4_attr $GITCO $OUTDIR/NPP_VIAES_L1.$H4DATESTRING.hdf /Data_Products/VIIRS-IMG-GEO-TC/VIIRS-IMG-GEO-TC_Aggr AggregateBeginningTime StartTime \"$STARTDATESTRING\""
eval "$PPDIR/hdf5_to_hdf4_attr $GITCO $OUTDIR/NPP_VIAES_L1.$H4DATESTRING.hdf /Data_Products/VIIRS-IMG-GEO-TC/VIIRS-IMG-GEO-TC_Aggr AggregateEndingTime EndTime \"$ENDDATESTRING\""
