/* 
	Copyright © 1999-2007, United States Government as represented by the 
	Administrator for The National Aeronautics and Space Administration.
	All Rights Reserved.
*/
/*
!C******************************************************************
!Description:
	Functions to handle input from HDF4 files.

!Revision History:

!References and Credits:
	W. Schroeder, University of Maryland
	W. Walsh, University of Maryland

!Design Notes:

!END*************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <unistd.h>
#include "const.h"
#include "types.h"
#include "input.h"
#include "error.h"
#include "misc.h"


static void unscale_bt(uint16 **, float **, double scale, double offset, int scan_lines);
static void unscale_ref(uint16 **, double scale, double offset, int scan_lines);


/*
* Purpose: Open all SDSs and store the indexs as well as scale/offsets
*
* Inputs:	in_info - InputInfo struct (input.h)
*
* Outputs: 	in_info - InputInfo struct (input.h)
*
* Returns 0 for success, otherwise non-zero error code.
*/
int InitInputHDF(InputInfo *in_info){
	int st;
	float buff[2];
	int32 dim_sizes[2];

	/* Get filenames without paths */
	char *temp_str;
	temp_str = malloc(256*sizeof(char));
	if ( !(in_info->viae_filename = malloc(256*sizeof(char))) )
		return error(ERR_F_MEM);
	if ( !(in_info->vmae_filename = malloc(256*sizeof(char))) )
		return error(ERR_F_MEM);
	if ( !(in_info->imft_filename = malloc(256*sizeof(char))) )
		return error(ERR_F_MEM);
	if ( !(in_info->qslwmip_filename = malloc(256*sizeof(char))) )
		return error(ERR_F_MEM);
	if ( !(in_info->vcdgip_filename  = malloc(256*sizeof(char))) )
		return error(ERR_F_MEM);

	strcpy(temp_str, in_info->infile_viae);
	strcpy(in_info->viae_filename, basename(temp_str));
	strcpy(temp_str, in_info->infile_vmae);
	strcpy(in_info->vmae_filename, basename(temp_str));
	strcpy(temp_str, in_info->infile_imft);
	strcpy(in_info->imft_filename, basename(temp_str));
	strcpy(temp_str, in_info->infile_qslwmip);
	strcpy(in_info->qslwmip_filename, basename(temp_str));

	if (in_info->infile_vcdgip != NULL){
		strcpy(temp_str, in_info->infile_vcdgip);
		strcpy(in_info->vcdgip_filename, basename(temp_str));
	}else{
		in_info->vcdgip_filename = NULL;
	}

	free(temp_str);
	

	/* Get file ids */
	if ((in_info->sd_id_viae = SDstart(in_info->infile_viae, DFACC_READ)) == FAIL) {
		printf("Unable to open HDF input file \"%s\".\n", in_info->infile_viae);
		exit(1);
	}

	if ((in_info->sd_id_imft = SDstart(in_info->infile_imft, DFACC_READ)) == FAIL) {
		printf("Unable to open HDF input file \"%s\".\n", in_info->infile_imft);
		exit(1);
	}

	if ((in_info->sd_id_vmae = SDstart(in_info->infile_vmae, DFACC_READ)) == FAIL) {
		printf("Unable to open HDF input file \"%s\".\n", in_info->infile_vmae);
		exit(1);
	}

	if ((in_info->sd_id_qslwmip = SDstart(in_info->infile_qslwmip, DFACC_READ)) == FAIL) {
		printf("Unable to open HDF input file \"%s\".\n", in_info->infile_qslwmip);
		exit(1);
	}


	if(in_info->infile_vcdgip != NULL){
		if ((in_info->sd_id_vcdgip = SDstart(in_info->infile_vcdgip, DFACC_READ)) == FAIL) {
			printf("Unable to open HDF input file \"%s\".\n", in_info->infile_vcdgip);
			exit(1);
		}
	}else{
		in_info->sd_id_vcdgip = NULL;
	}

	/* MidTime -- only for hdf4 version */
	if ((in_info->h_id_imft = Hopen(in_info->infile_imft, DFACC_READ, 0)) == FAIL) {
			printf("Unable to open MidTime from input file \"%s\".\n", in_info->infile_imft);
			exit(1);
	}


	/* ---------- I-bands ---------- */

	/* I1 reflectance */
	if ( (st = init_input_sds(in_info->sd_id_viae, "Reflectance_I1",
		&in_info->sds_index_ref1, DFNT_UINT16, dim_sizes,
		&in_info->ScaleR1, &in_info->OffsetR1, 1)) ) return error(st);

	/* Find number of scans */
	in_info->nscans = dim_sizes[0] / 32;
 
	/* I1 QF */ 
	if ( (st = init_input_sds(in_info->sd_id_viae, "QF1_VIIRSIBANDSDR_I1",
		&in_info->sds_index_QF1, DFNT_UINT8, dim_sizes,
		NULL, NULL, 0)) ) return error(st);


	/* I2 reflectance */
	if ( (st = init_input_sds(in_info->sd_id_viae, "Reflectance_I2",
		&in_info->sds_index_ref2, DFNT_UINT16, dim_sizes,
		&in_info->ScaleR2, &in_info->OffsetR2, 1)) ) return error(st);

	/* I2 QF */
	if ( (st = init_input_sds(in_info->sd_id_viae, "QF1_VIIRSIBANDSDR_I2",
		&in_info->sds_index_QF2, DFNT_UINT8, dim_sizes,
		NULL, NULL, 0)) ) return error(st);

	/* I3 reflectance */
	if ( (st = init_input_sds(in_info->sd_id_viae, "Reflectance_I3",
		&in_info->sds_index_ref3, DFNT_UINT16, dim_sizes,
		&in_info->ScaleR3, &in_info->OffsetR3, 1)) ) return error(st);

	/* I3 QF */
	if ( (st = init_input_sds(in_info->sd_id_viae, "QF1_VIIRSIBANDSDR_I3",
		&in_info->sds_index_QF3, DFNT_UINT8, dim_sizes,
		NULL, NULL, 0)) ) return error(st);

	/* I4 brightness tempature */
	if ( (st = init_input_sds(in_info->sd_id_viae, "BrightnessTemperature_I4",
		&in_info->sds_index_bt4, DFNT_UINT16, dim_sizes,
		&in_info->ScaleT4, &in_info->OffsetT4, 1)) ) return error(st);

	/* I4 QF */
	if ( (st = init_input_sds(in_info->sd_id_viae, "QF1_VIIRSIBANDSDR_I4",
		&in_info->sds_index_QF4, DFNT_UINT8, dim_sizes,
		NULL, NULL, 0)) ) return error(st);

	/* I5 brightness tempature */
	if ( (st = init_input_sds(in_info->sd_id_viae, "BrightnessTemperature_I5",
		&in_info->sds_index_bt5, DFNT_UINT16, dim_sizes,
		&in_info->ScaleT5, &in_info->OffsetT5, 1)) ) return error(st);

	/* I5 QF */
	if ( (st = init_input_sds(in_info->sd_id_viae, "QF1_VIIRSIBANDSDR_I5",
		&in_info->sds_index_QF5, DFNT_UINT8, dim_sizes,
		NULL, NULL, 0)) ) return error(st);



	/* ---------- M-bands ---------- */

	/* M13 radiance */
	if ( (st = init_input_sds(in_info->sd_id_vmae, "Radiance_M13",
		&in_info->sds_index_rad13_agg, DFNT_FLOAT32, dim_sizes,
		NULL, NULL, 0)) ) return error(st);
	/* M13 QF */
	if ( (st = init_input_sds(in_info->sd_id_vmae, "QF1_VIIRSMBANDSDR_M13",
		&in_info->sds_index_QF13, DFNT_UINT8, dim_sizes,
		NULL, NULL, 0)) ) return error(st);

	/* Unagg M13 radiance */
	if(in_info->sd_id_vcdgip != NULL){
		if ( (st = init_input_sds(in_info->sd_id_vcdgip, "radiance_6",
			&in_info->sds_index_rad13_unagg, DFNT_FLOAT32, dim_sizes,
			NULL, NULL, 0)) ) return error(st);

	}else{
		in_info->sds_index_rad13_unagg = NULL;
	}


	/* ---------- Geolocation ---------- */
	if ( (st = init_input_sds(in_info->sd_id_imft, "Latitude",
		&in_info->sds_index_lat, DFNT_FLOAT32, dim_sizes,
		NULL, NULL, 0)) ) return error(st);

	if ( (st = init_input_sds(in_info->sd_id_imft, "Longitude",
		&in_info->sds_index_lon, DFNT_FLOAT32, dim_sizes,
		NULL, NULL, 0)) ) return error(st);

	if ( (st = init_input_sds(in_info->sd_id_imft, "SolarZenithAngle",
		&in_info->sds_index_solzen, DFNT_FLOAT32, dim_sizes,
		NULL, NULL, 0)) ) return error(st);

	if ( (st = init_input_sds(in_info->sd_id_imft, "SolarAzimuthAngle",
		&in_info->sds_index_solaz, DFNT_FLOAT32, dim_sizes,
		NULL, NULL, 0)) ) return error(st);

	if ( (st = init_input_sds(in_info->sd_id_imft, "SatelliteZenithAngle",
		&in_info->sds_index_viewzen, DFNT_FLOAT32, dim_sizes,
		NULL, NULL, 0)) ) return error(st);

	if ( (st = init_input_sds(in_info->sd_id_imft, "SatelliteAzimuthAngle",
		&in_info->sds_index_viewaz, DFNT_FLOAT32, dim_sizes,
		NULL, NULL, 0)) ) return error(st);

        if ( (st = init_input_sds(in_info->sd_id_imft, "QF2_VIIRSSDRGEO",
		&in_info->sds_index_QFGeo, DFNT_UINT8, dim_sizes,
		NULL, NULL, 0)) ) return error(st);

	// DRL: Get MidTime too, as a 64-bit float SDS
	int32 mt_dim_sizes[1];
	if ( (st = init_input_sds_1d(in_info->sd_id_imft, "MidTime",
		&in_info->sds_index_midtime, DFNT_FLOAT64, mt_dim_sizes,
		NULL, NULL, 0)) ) return error(st);

	/* Get Beginning Date/Time */
	int attr_index = SDfindattr(in_info->sd_id_imft, "BeginningDate");
	if (attr_index != -1){
		SDreadattr(in_info->sd_id_imft, attr_index, &in_info->BeginningDate);
		in_info->BeginningDate[8] = '\0';
	}

	attr_index = SDfindattr(in_info->sd_id_imft, "BeginningTime");
	if (attr_index != -1){
		SDreadattr(in_info->sd_id_imft, attr_index, &in_info->BeginningTime);
		in_info->BeginningTime[14] = '\0';
	}

	/* ---------- Land Water Mask ---------- */
	if ( (st = init_input_sds(in_info->sd_id_qslwmip, "QSTLWM",
		&in_info->sds_index_qstlwm, DFNT_UINT8, dim_sizes,
		NULL, NULL, 0)) ) return error(st);

	return 0;
}


/*
* Purpose: Open SDS and call read_scale_offset
*
* Inputs:	sd_id
*		sds_name
*		expected_data_type
*		read_factors - 1 = read scale/offset
*
* Outputs: 	sds_index_ptr - InputInfo struct (input.h)
*		dim_sizes
*		scale
*		offset
*
* Returns 0 for success, otherwise non-zero error code.
*/
int init_input_sds(int32 sd_id, char *sds_name, int32 *sds_index_ptr,
	int32 expected_data_type, int32 dim_sizes[MAX_VAR_DIMS],
	float32 *scale, float32 *offset, int read_factors)
{
	int32 sds_index, sds_id, rank, data_type, nattr;
	char dummy_name[MAX_NC_NAME];
	int st;

	if ((sds_index = SDnametoindex(sd_id, sds_name)) == FAIL)
		return 1;

	if ((sds_id = SDselect(sd_id, sds_index)) == FAIL) return 1;

	if (SDgetinfo(sds_id, dummy_name, &rank, dim_sizes, &data_type,
		&nattr) == FAIL) return 1;

	/* check rank and data type */
	if (rank != 2) return 2;
	if (data_type != expected_data_type) return 2;

	if (read_factors) {
		if ( (st = read_scale_offset(sds_id, scale, offset)) ) return st;
	}

	if (SDendaccess(sds_id) == FAIL) return 2;

	*sds_index_ptr = sds_index;
	return 0;
}

/* DRL: Function to initialize a one-dimensional HDF4 SDS */
int init_input_sds_1d(int32 sd_id, char *sds_name, int32 *sds_index_ptr,
	int32 expected_data_type, int32 dim_sizes[1],
	float32 *scale, float32 *offset, int read_factors)
{
	int32 sds_index, sds_id, rank, data_type, nattr;
	char dummy_name[MAX_NC_NAME];
	int st;

	if ((sds_index = SDnametoindex(sd_id, sds_name)) == FAIL){
		printf("SDnametoindex failed for 1d");
		return 1;
	}

	if ((sds_id = SDselect(sd_id, sds_index)) == FAIL){
		printf("SDselect failed for 1d");
		return 1;
	}

	if (SDgetinfo(sds_id, dummy_name, &rank, dim_sizes, &data_type,
		&nattr) == FAIL){
		printf("SDgetinfo failed for 1d");
		return 1;
	}

	/* check data type */
	if (data_type != expected_data_type){
		printf("Datatype mismatch for 1d");
		return 2;
	}

	if (read_factors) {
		if ( (st = read_scale_offset(sds_id, scale, offset)) ){
			printf("read_scale_offset failed for 1d");
			return st;
		}
	}

	if (SDendaccess(sds_id) == FAIL){
		printf("SDendaccess failed for 1d"); 
		return 2;
	}

	*sds_index_ptr = sds_index;
	return 0;
}

/*
* Purpose: Read scale/offset
*
* Inputs:	sds_id
*
* Outputs:	scale
*		offset
*
* Returns 0 for success, otherwise non-zero error code.
*/
int read_scale_offset(int32 sds_id, float32 *scale, float32 *offset){
	int32 attr_index;
	int32 n_values;
	int32 data_type;
	char attr_name[MAX_NC_NAME];

	if ((attr_index = SDfindattr(sds_id, "Scale")) == FAIL) {
		scale = -999;
		offset = -999;
		return 0;
	}else {

		if (SDattrinfo(sds_id, attr_index, attr_name, &data_type,
			&n_values)) return 2;

		if (data_type != DFNT_FLOAT32) return 2;

		if (SDreadattr(sds_id, attr_index, scale)) return 2;
		}


	if ((attr_index = SDfindattr(sds_id, "Offset")) == FAIL) {

		return 1;
	}else {

		if (SDattrinfo(sds_id, attr_index, attr_name, &data_type,
			&n_values)) return 2;

		if (data_type != DFNT_FLOAT32) return 2;
		if (SDreadattr(sds_id, attr_index, (void *) offset))
			return 2;
		}

	return 0;
}


/*
* Purpose: Reads all the input SDSs and if needed unscales them
*
* Inputs:	in_info - InputInfo struct (input.h)
*
* Outputs: 	jugg - Juggernaut struct (types.h)
*
* Returns 0 for success, otherwise non-zero error code.
*/
int ReadInputs(InputInfo *in_info, Juggernaut *jugg){	

	int i, j, st, sds_id;
	int start[2], edges[2];

	start[0] = 0;
	edges[0] = jugg->number_of_scan_lines;

	start[1] = 0;
	edges[1] = SAMPLES_375M;

	/* buff is used to get scaled reflectance data */
	uint16 **buff;
	if ( !(buff = (uint16**)malloc(jugg->number_of_scan_lines*sizeof(uint16*))) )
		return error(ERR_F_MEM);
	if ( !(buff[0] = (uint16*)malloc(jugg->number_of_scan_lines*SAMPLES_375M*sizeof(uint16))) )
		return error(ERR_F_MEM);
	for (i=1;i<jugg->number_of_scan_lines;i++){
		buff[i] = buff[i-1] + SAMPLES_375M;
	}

	/* ----- I-band ----- */
	if(in_info->ScaleR1 != -999){
		/* I1 reflectance */
		if ( !(jugg->R1 = (uint16**)malloc(jugg->number_of_scan_lines*sizeof(uint16*))) )
			return error(ERR_F_MEM);
		if ( !(jugg->R1[0] = (uint16*)malloc(jugg->number_of_scan_lines*SAMPLES_375M*sizeof(uint16))) )
			return error(ERR_F_MEM);
		for (i=1;i<jugg->number_of_scan_lines;i++){
			jugg->R1[i] = jugg->R1[i-1] + SAMPLES_375M;
		}

		/* Read */
		if ((sds_id = SDselect(in_info->sd_id_viae, in_info->sds_index_ref1)) == FAIL) return ERR_HDF;

		if (SDreaddata(sds_id, start, NULL, edges, jugg->R1[0]) == FAIL)
			return ERR_HDF;

		/* Scale */
		unscale_ref(jugg->R1, in_info->ScaleR1, in_info->OffsetR1, jugg->number_of_scan_lines);

		/* I1 QA */
		if ( !(jugg->QF1 = (uint8**)malloc(jugg->number_of_scan_lines*sizeof(uint8*))) )
			return error(ERR_F_MEM);
		if ( !(jugg->QF1[0] = (uint8*)malloc(jugg->number_of_scan_lines*SAMPLES_375M*sizeof(uint8))) )
			return error(ERR_F_MEM);
		for (i=1;i<jugg->number_of_scan_lines;i++){
			jugg->QF1[i] = jugg->QF1[i-1] + SAMPLES_375M;
		}
	
		/* Read */
		if ((sds_id = SDselect(in_info->sd_id_viae, in_info->sds_index_QF1)) == FAIL) return ERR_HDF;
		if (SDreaddata(sds_id, start, NULL, edges, jugg->QF1[0]) == FAIL)
			return ERR_HDF;
	}

	if(in_info->ScaleR2 != -999){
		/* I2 reflectance */
		if ( !(jugg->R2 = (uint16**)malloc(jugg->number_of_scan_lines*sizeof(uint16*))) )
			return error(ERR_F_MEM);
		if ( !(jugg->R2[0] = (uint16*)malloc(jugg->number_of_scan_lines*SAMPLES_375M*sizeof(uint16))) )
			return error(ERR_F_MEM);
		for (i=1;i<jugg->number_of_scan_lines;i++){
			jugg->R2[i] = jugg->R2[i-1] + SAMPLES_375M;
		}

		/* Read */
		if ((sds_id = SDselect(in_info->sd_id_viae, in_info->sds_index_ref2)) == FAIL) return ERR_HDF;
		if (SDreaddata(sds_id, start, NULL, edges, jugg->R2[0]) == FAIL)
			return ERR_HDF;

		/* Scale */
		unscale_ref(jugg->R2, in_info->ScaleR2, in_info->OffsetR2, jugg->number_of_scan_lines);

		/* I2 QA */
		if ( !(jugg->QF2 = (uint8**)malloc(jugg->number_of_scan_lines*sizeof(uint8*))) )
			return error(ERR_F_MEM);
		if ( !(jugg->QF2[0] = (uint8*)malloc(jugg->number_of_scan_lines*SAMPLES_375M*sizeof(uint8))) )
			return error(ERR_F_MEM);
		for (i=1;i<jugg->number_of_scan_lines;i++){
			jugg->QF2[i] = jugg->QF2[i-1] + SAMPLES_375M;
		}
	
		/* Read */
		if ((sds_id = SDselect(in_info->sd_id_viae, in_info->sds_index_QF2)) == FAIL) return ERR_HDF;
		if (SDreaddata(sds_id, start, NULL, edges, jugg->QF2[0]) == FAIL)
			return ERR_HDF;
	}

	if(in_info->ScaleR3 != -999){
		/* I3 reflectance */
		if ( !(jugg->R3 = (uint16**)malloc(jugg->number_of_scan_lines*sizeof(uint16*))) )
			return error(ERR_F_MEM);
		if ( !(jugg->R3[0] = (uint16*)malloc(jugg->number_of_scan_lines*SAMPLES_375M*sizeof(uint16))) )
			return error(ERR_F_MEM);
		for (i=1;i<jugg->number_of_scan_lines;i++){
			jugg->R3[i] = jugg->R3[i-1] + SAMPLES_375M;
		}

		/* Read */
		if ((sds_id = SDselect(in_info->sd_id_viae, in_info->sds_index_ref3)) == FAIL) return ERR_HDF;
		if (SDreaddata(sds_id, start, NULL, edges, jugg->R3[0]) == FAIL)
			return ERR_HDF;
		/* Scale */
		unscale_ref(jugg->R3, in_info->ScaleR3, in_info->OffsetR3, jugg->number_of_scan_lines);

		/* I3 QA */
		if ( !(jugg->QF3 = (uint8**)malloc(jugg->number_of_scan_lines*sizeof(uint8*))) )
			return error(ERR_F_MEM);
		if ( !(jugg->QF3[0] = (uint8*)malloc(jugg->number_of_scan_lines*SAMPLES_375M*sizeof(uint8))) )
			return error(ERR_F_MEM);
		for (i=1;i<jugg->number_of_scan_lines;i++){
			jugg->QF3[i] = jugg->QF3[i-1] + SAMPLES_375M;
		}
	
		/* Read */
		if ((sds_id = SDselect(in_info->sd_id_viae, in_info->sds_index_QF3)) == FAIL) return ERR_HDF;
		if (SDreaddata(sds_id, start, NULL, edges, jugg->QF3[0]) == FAIL)
			return ERR_HDF;
	}	

	/* I4 brightness tempature */
	if ( !(jugg->T4 = (float**)malloc(jugg->number_of_scan_lines*sizeof(float*))) )
		return error(ERR_F_MEM);
	if ( !(jugg->T4[0] = (float*)malloc(jugg->number_of_scan_lines*SAMPLES_375M*sizeof(float))) )
		return error(ERR_F_MEM);
	for (i=1;i<jugg->number_of_scan_lines;i++){
		jugg->T4[i] = jugg->T4[i-1] + SAMPLES_375M;
	}

	/* Read */
	if ((sds_id = SDselect(in_info->sd_id_viae, in_info->sds_index_bt4)) == FAIL) return ERR_HDF;
	if (SDreaddata(sds_id, start, NULL, edges, buff[0]) == FAIL)
		return ERR_HDF;
	/* Scale */
	unscale_bt(buff, jugg->T4, in_info->ScaleT4, in_info->OffsetT4, jugg->number_of_scan_lines);


	/* I4 QA */
	if ( !(jugg->QF4 = (uint8**)malloc(jugg->number_of_scan_lines*sizeof(uint8*))) )
		return error(ERR_F_MEM);
	if ( !(jugg->QF4[0] = (uint8*)malloc(jugg->number_of_scan_lines*SAMPLES_375M*sizeof(uint8))) )
		return error(ERR_F_MEM);
	for (i=1;i<jugg->number_of_scan_lines;i++){
		jugg->QF4[i] = jugg->QF4[i-1] + SAMPLES_375M;
	}
	
	/* Read */
	if ((sds_id = SDselect(in_info->sd_id_viae, in_info->sds_index_QF4)) == FAIL) return ERR_HDF;
	if (SDreaddata(sds_id, start, NULL, edges, jugg->QF4[0]) == FAIL)
		return ERR_HDF;

	/* I5 brightness tempature */
	if ( !(jugg->T5 = (float**)malloc(jugg->number_of_scan_lines*sizeof(float*))) )
		return error(ERR_F_MEM);
	if ( !(jugg->T5[0] = (float*)malloc(jugg->number_of_scan_lines*SAMPLES_375M*sizeof(float))) )
		return error(ERR_F_MEM);
	for (i=1;i<jugg->number_of_scan_lines;i++){
		jugg->T5[i] = jugg->T5[i-1] + SAMPLES_375M;
	}

	/* Read */
	if ((sds_id = SDselect(in_info->sd_id_viae, in_info->sds_index_bt5)) == FAIL) return ERR_HDF;
	if (SDreaddata(sds_id, start, NULL, edges, buff[0]) == FAIL)
		return ERR_HDF;
	/* Scale */
	unscale_bt(buff, jugg->T5, in_info->ScaleT5, in_info->OffsetT5, jugg->number_of_scan_lines);

	/* I5 QA */
	if ( !(jugg->QF5 = (uint8**)malloc(jugg->number_of_scan_lines*sizeof(uint8*))) )
		return error(ERR_F_MEM);
	if ( !(jugg->QF5[0] = (uint8*)malloc(jugg->number_of_scan_lines*SAMPLES_375M*sizeof(uint8))) )
		return error(ERR_F_MEM);
	for (i=1;i<jugg->number_of_scan_lines;i++){
		jugg->QF5[i] = jugg->QF5[i-1] + SAMPLES_375M;
	}
	
	/* Read */
	if ((sds_id = SDselect(in_info->sd_id_viae, in_info->sds_index_QF5)) == FAIL) return ERR_HDF;
	if (SDreaddata(sds_id, start, NULL, edges, jugg->QF5[0]) == FAIL)
		return ERR_HDF;

	/* ----- I-band Geo ----- */
	/* latitude */
	if ( !(jugg->lat = (float**)malloc(jugg->number_of_scan_lines*sizeof(float*))) )
		return error(ERR_F_MEM);
	if ( !(jugg->lat[0] = (float*)malloc(jugg->number_of_scan_lines*SAMPLES_375M*sizeof(float))) )
		return error(ERR_F_MEM);
	for (i=1;i<jugg->number_of_scan_lines;i++){
		jugg->lat[i] = jugg->lat[i-1] + SAMPLES_375M;
	}

	/* Read */
	if ((sds_id = SDselect(in_info->sd_id_imft, in_info->sds_index_lat)) == FAIL) return ERR_HDF;
	if (SDreaddata(sds_id, start, NULL, edges, jugg->lat[0]) == FAIL)
		return ERR_HDF;
	
	/* longitude */
	if ( !(jugg->lon = (float**)malloc(jugg->number_of_scan_lines*sizeof(float*))) )
		return error(ERR_F_MEM);
	if ( !(jugg->lon[0] = (float*)malloc(jugg->number_of_scan_lines*SAMPLES_375M*sizeof(float))) )
		return error(ERR_F_MEM);
	for (i=1;i<jugg->number_of_scan_lines;i++){
		jugg->lon[i] = jugg->lon[i-1] + SAMPLES_375M;
	}

	/* Read */
	if ((sds_id = SDselect(in_info->sd_id_imft, in_info->sds_index_lon)) == FAIL) return ERR_HDF;
	if (SDreaddata(sds_id, start, NULL, edges, jugg->lon[0]) == FAIL)
		return ERR_HDF;

	/* view zenith angle */
	if ( !(jugg->ViewZenAng = (float**)malloc(jugg->number_of_scan_lines*sizeof(float*))) )
		return error(ERR_F_MEM);
	if ( !(jugg->ViewZenAng[0] = (float*)malloc(jugg->number_of_scan_lines*SAMPLES_375M*sizeof(float))) )
		return error(ERR_F_MEM);
	for (i=1;i<jugg->number_of_scan_lines;i++){
		jugg->ViewZenAng[i] = jugg->ViewZenAng[i-1] + SAMPLES_375M;
	}

	/* Read */
	if ((sds_id = SDselect(in_info->sd_id_imft, in_info->sds_index_viewzen)) == FAIL) return ERR_HDF;
	if (SDreaddata(sds_id, start, NULL, edges, jugg->ViewZenAng[0]) == FAIL)
		return ERR_HDF;

	/* solar zenith angle */
	if ( !(jugg->SolZenAng = (float**)malloc(jugg->number_of_scan_lines*sizeof(float*))) )
		return error(ERR_F_MEM);
	if ( !(jugg->SolZenAng[0] = (float*)malloc(jugg->number_of_scan_lines*SAMPLES_375M*sizeof(float))) )
		return error(ERR_F_MEM);
	for (i=1;i<jugg->number_of_scan_lines;i++){
		jugg->SolZenAng[i] = jugg->SolZenAng[i-1] + SAMPLES_375M;
	}

	/* Read */
	if ((sds_id = SDselect(in_info->sd_id_imft, in_info->sds_index_solzen)) == FAIL) return ERR_HDF;
	if (SDreaddata(sds_id, start, NULL, edges, jugg->SolZenAng[0]) == FAIL)
		return ERR_HDF;
	
	/* view azimuth angle */
	if ( !(jugg->ViewAzAng = (float**)malloc(jugg->number_of_scan_lines*sizeof(float*))) )
		return error(ERR_F_MEM);
	if ( !(jugg->ViewAzAng[0] = (float*)malloc(jugg->number_of_scan_lines*SAMPLES_375M*sizeof(float))) )
		return error(ERR_F_MEM);
	for (i=1;i<jugg->number_of_scan_lines;i++){
		jugg->ViewAzAng[i] = jugg->ViewAzAng[i-1] + SAMPLES_375M;
	}

	/* Read */
	if ((sds_id = SDselect(in_info->sd_id_imft, in_info->sds_index_viewaz)) == FAIL) return ERR_HDF;
	if (SDreaddata(sds_id, start, NULL, edges, jugg->ViewAzAng[0]) == FAIL)
		return ERR_HDF;

	/* solar azimuth angle */
	if ( !(jugg->SolAzAng = (float**)malloc(jugg->number_of_scan_lines*sizeof(float*))) )
		return error(ERR_F_MEM);
	if ( !(jugg->SolAzAng[0] = (float*)malloc(jugg->number_of_scan_lines*SAMPLES_375M*sizeof(float))) )
		return error(ERR_F_MEM);
	for (i=1;i<jugg->number_of_scan_lines;i++){
		jugg->SolAzAng[i] = jugg->SolAzAng[i-1] + SAMPLES_375M;
	}

	/* Read */
	if ((sds_id = SDselect(in_info->sd_id_imft, in_info->sds_index_solaz)) == FAIL) return ERR_HDF;
	if (SDreaddata(sds_id, start, NULL, edges, jugg->SolAzAng[0]) == FAIL)
		return ERR_HDF;

	/* DRL: mid time as float64 SDS */
	int32 mtStart[1], mtEdges[1];
	mtStart[0] = 0;
	mtEdges[0] = (int32)(jugg->number_of_scan_lines / 32);

	if ( !( jugg->MidTime = (float64*)malloc(mtEdges[0] * sizeof(float64)) ) )
		return error(ERR_F_MEM);

	/* DRL: read mid time */
	if ((sds_id = SDselect(in_info->sd_id_imft, in_info->sds_index_midtime)) == FAIL){
		printf("SDselect for MidTime failed!\n");
		return ERR_HDF;
	}
	if (SDreaddata(sds_id, mtStart, NULL, mtEdges, jugg->MidTime) == FAIL){
		printf("SDreaddata for MidTime failed!\n");
		return ERR_HDF;
	}

	/* mid time */
	/*
	int32 n_records, vdata_size;
	st = Vstart(in_info->h_id_imft);

	int ref = VSfind(in_info->h_id_imft, "MidTime");
	int vdata_id = VSattach(in_info->h_id_imft, ref, "r");
	st = VSinquire(vdata_id, &n_records, NULL, NULL, &vdata_size, NULL);

	int bufsz = n_records*sizeof(float64);
	uint8 databuf[n_records*sizeof(float64)];
	VOIDP ptr[1];
	if ( !(jugg->MidTime = (float64*)malloc(n_records*sizeof(float64))) )
		return error(ERR_F_MEM);

	st = VSsetfields(vdata_id, "MidTime");
	st = VSread(vdata_id, databuf, n_records, 1);

	ptr[0] = &jugg->MidTime[0];
   	st = VSfpack(vdata_id, _HDF_VSUNPACK, "MidTime", databuf, bufsz, n_records, "MidTime", ptr);
	*/
	/*** 		***/

	/* qa */
	if ( !(jugg->QFGeo = (uint8**)malloc(jugg->number_of_scan_lines*sizeof(uint8*))) )
		return error(ERR_F_MEM);
	if ( !(jugg->QFGeo[0] = (uint8*)malloc(jugg->number_of_scan_lines*SAMPLES_375M*sizeof(uint8))) )
		return error(ERR_F_MEM);
	for (i=1;i<jugg->number_of_scan_lines;i++){
		jugg->QFGeo[i] = jugg->QFGeo[i-1] + SAMPLES_375M;
	}
	
	/* Read */
	if ((sds_id = SDselect(in_info->sd_id_imft, in_info->sds_index_QFGeo)) == FAIL) return ERR_HDF;
	if (SDreaddata(sds_id, start, NULL, edges, jugg->QFGeo[0]) == FAIL)
		return ERR_HDF;

	/* M-Band */
	int m_scan_lines = jugg->number_of_scan_lines / 2;

	start[0] = 0;
	edges[0] = m_scan_lines;

	start[1] = 0;
	edges[1] = SAMPLES_750M;

	if ( !(jugg->Rad13_agg = (float**)malloc(m_scan_lines*sizeof(float*))) )
		return error(ERR_F_MEM);
	if ( !(jugg->Rad13_agg[0] = (float*)malloc(m_scan_lines*SAMPLES_750M*sizeof(float))) )
		return error(ERR_F_MEM);
	for (i=1;i<m_scan_lines;i++){
		jugg->Rad13_agg[i] = jugg->Rad13_agg[i-1] + SAMPLES_750M;
	}

	if ((sds_id = SDselect(in_info->sd_id_vmae, in_info->sds_index_rad13_agg)) == FAIL) return ERR_HDF;
	if (SDreaddata(sds_id, start, NULL, edges, jugg->Rad13_agg[0]) == FAIL)
		return ERR_HDF;

	/* M13 QA -- wrong float? */
	if ( !(jugg->QF13 = (float**)malloc(m_scan_lines*sizeof(float*))) )
		return error(ERR_F_MEM);
	if ( !(jugg->QF13[0] = (float*)malloc(m_scan_lines*SAMPLES_750M*sizeof(float))) )
		return error(ERR_F_MEM);
	for (i=1;i<m_scan_lines;i++){
		jugg->QF13[i] = jugg->QF13[i-1] + SAMPLES_750M;
	}


	if ((sds_id = SDselect(in_info->sd_id_vmae, in_info->sds_index_QF13)) == FAIL) return ERR_HDF;
	if (SDreaddata(sds_id, start, NULL, edges, jugg->QF13[0]) == FAIL)
		return ERR_HDF;

	/* ----- Land Water Mask ----- */
	if ( !(jugg->LWM = (uint8**)malloc(m_scan_lines*sizeof(uint8*))) )
		return error(ERR_F_MEM);
	if ( !(jugg->LWM[0] = (uint8*)malloc(m_scan_lines*SAMPLES_750M*sizeof(uint8))) )
		return error(ERR_F_MEM);
	for (i=1;i<m_scan_lines;i++){
		jugg->LWM[i] = jugg->LWM[i-1] + SAMPLES_750M;
	}

	if ((sds_id = SDselect(in_info->sd_id_qslwmip, in_info->sds_index_qstlwm)) == FAIL) return ERR_HDF;
	if (SDreaddata(sds_id, start, NULL, edges, jugg->LWM[0]) == FAIL)
		return ERR_HDF;

	/* Free Buff */
	free(buff[0]);
	free(buff);

	return 0;
}



/*
* Purpose: Unscale scaled brightness temperatures into physical values.
*	   Missing data will be flagged with a value of -1.
*
* Inputs:	ScaledVal
*		scale
*		offset
*		scan_lines
*
* Outputs: 	UnscaledVal
*
* Returns 0 for success, otherwise non-zero error code.
*/
static void unscale_bt(uint16 **ScaledVal, float **UnscaledVal, double scale, double offset, int scan_lines){
	int i, j;
	const float missval = 0;

	for(i=0;i<scan_lines;i++){
		for(j=0;j<SAMPLES_375M;j++){
			if (ScaledVal[i][j] < 65528){
				UnscaledVal[i][j] = ScaledVal[i][j]*scale + offset;
			}else{
				UnscaledVal[i][j] = missval;
			}		
		}
	}
}


/*
* Purpose: Unscale scaled reflectances into physical values.
*	   Missing data will be flagged with a value of -1.
*
* Inputs:	ScaledVal
*		scale
*		offset
*		scan_lines
*
* Outputs: 	UnscaledVal
*
* Returns 0 for success, otherwise non-zero error code.
*/
static void unscale_ref(uint16 **Ref, double scale, double offset, int scan_lines){
	int i, j;
	const float missval = 0;

	for(i=0;i<scan_lines;i++){
		for(j=0;j<SAMPLES_375M;j++){
			if (Ref[i][j] < 65528){
				Ref[i][j] = (int) trunc(roundf(10000 * (Ref[i][j]*scale + offset)));
			}else{
				Ref[i][j] = missval;
			}		
		}
	}
}
















