/* 
	Copyright © 1999-2007, United States Government as represented by the 
	Administrator for The National Aeronautics and Space Administration.
	All Rights Reserved.
*/

#include <stdlib.h>
#include <libgen.h>
#include "mfhdf.h"
#include "hdf5.h"
#define MAX_DATASETS 50
#define MAX_DATASETNAME 500
#define MAXLENGTH 512
#define FALSE 0
#define DEBUG

/**
 * DRL utility to transfer HDF5 attributes to HDF4 files as global attributes. Based on the
 * code from the "h5SDS_transfer_rename" utility.
 *
 */
int main(int argc, char **argv){
	/* 
	* Argument checks:
	*	argv[1] = Path to HDF5 file
	*	argv[2] = Path to HDF4 file
	*	argv[3] = Full path to group/dataset/datatype in HDF5 file
	*	argv[4] = HDF5 attribute name
	*	argv[5] = HDF4 attribute name
	*	argv[6] = (optional) override value for attribute; type determined by source attribute
	*	* Note: as of 2.5.1, override value only works properly for string attributes!
	*/
	if (argc < 6) {
		printf("\nProgram for reading an attribute from an HDF5 file and copying it to an HDF4 file as a top-level attribute -- with renaming\n");
		printf("USAGE: %s <Source HDF5 file> <Destination HDF4 file> <Full path to attr in source> <source attr name> <dest attr name> <override value>\n", argv[0]);
		exit(0);
	}

	/* Declare HDF5 variables */
	FILE *fd;
	hid_t h5file, h5dataset, h5attrid, h5attrdataspace, h5attrdatatype;
	int h5attrrank;
	hsize_t* h5attrcurrent_dims;
	hsize_t* h5attrmax_dims;

	/* Declare HDF4 variables */
	int32 h4file;

	/* Intended to be a generic data buffer, which accomodates multiple HDF5/4 datatypes */
	int buf_size;
	int genbuf[10000];

	/* Declare functions */
	void write_local_SDS_attribute(int32 sds_out, int buf_size, char * name, int * big_array, 
                               ssize_t ord, ssize_t sign, ssize_t size, ssize_t class);

	/* Open the input file for read, which is an HDF5 file */
	if( (h5file = H5Fopen(argv[1], H5F_ACC_RDONLY, H5P_DEFAULT)) == -1){
		printf("ERROR: H5Fopen() failed on file '%s'\n", argv[1]);
		H5Fclose(h5file);
		exit(-1);
	}

	/* Open the HDF5 named datatype that contains the attribute */
	if( (h5dataset = H5Dopen1(h5file, argv[3])) < 0 ){
		printf("ERROR: H5Dopen() failed on dataset '%s'\n", argv[3]);
		H5Dclose(h5dataset);
		H5Fclose(h5file);
		exit(-1);
	}	

	/* Open the HDF5 attribute */
	if( (h5attrid = H5Aopen_by_name(h5dataset, argv[3], argv[4], H5P_DEFAULT, H5P_DEFAULT)) < 0 ){
		printf("ERROR: H5Aopen_by_name() failed on attr '%s'\n", argv[4]);
		H5Aclose(h5attrid);
		H5Dclose(h5dataset);
		H5Fclose(h5file);
		exit(-1);
	}

	/* Get the HDF5 attribute's specs */
	h5attrdataspace = H5Aget_space(h5attrid);
	h5attrdatatype = H5Aget_type(h5attrid);
	h5attrrank = H5Sget_simple_extent_ndims(h5attrdataspace);

	/* Open the output file for write, which is an HDF4 file */
	if ((h4file = SDstart(argv[2], DFACC_WRITE)) == -1) {
		printf("ERROR: SDstart() failed on file '%s'\n", argv[2]);
		SDendaccess(h4file);
		SDend(h4file);
		exit(-1);
	}

	/* Set rank to 1 if zero, so we don't end up with zero memory allocations */
	if(h5attrrank == 0) h5attrrank = 1;
	printf("H5 attribute rank: %d\n", h5attrrank);

	/* Allocate memory for the "dimension" arrays */
	h5attrcurrent_dims = (hsize_t *)malloc(h5attrrank * sizeof(hsize_t));
	h5attrmax_dims = (hsize_t *)malloc(h5attrrank * sizeof(hsize_t));

	/* this assumes dataspace was created with H5Screate_simple() */
	H5Sget_simple_extent_dims(h5attrdataspace, h5attrcurrent_dims, h5attrmax_dims);

	/* read the hdf5 attribute's contents into the generic data buffer */
	H5Aread(h5attrid, h5attrdatatype, genbuf);

	/* then write it to the HDF4 file, use argv[5] as the name of the attribute */
	/* 2.5.1: if optional override value is provided, write that as the attribute's value instead; use
	   the source attribute's properties for this override value */
	if(argc > 6){
		write_local_SDS_attribute(h4file, h5attrmax_dims[0], argv[5], (int*)argv[6],
									H5Tget_order(h5attrdatatype), 
									H5Tget_sign(h5attrdatatype),
									H5Tget_size(h5attrdatatype),
									H5Tget_class(h5attrdatatype));
	}
	else {
		write_local_SDS_attribute(h4file, h5attrmax_dims[0], argv[5], genbuf,
									H5Tget_order(h5attrdatatype), 
									H5Tget_sign(h5attrdatatype),
									H5Tget_size(h5attrdatatype),
									H5Tget_class(h5attrdatatype));
	}
 
	/* clean up allocated arrays */
	free(h5attrmax_dims);
	free(h5attrcurrent_dims);

	/* Operation complete; clean up */
	H5Tclose(h5attrdatatype);
	H5Sclose(h5attrdataspace); 
	H5Aclose(h5attrid);
	H5Dclose(h5dataset);
	H5Fclose(h5file);
	SDendaccess(h4file);
	SDend(h4file);

	printf("Done\n");

	/* DRL: For some reason it was observed that this program would exit with return
	   code 5 even on successful results. This is added to counteract this behavior: */
	exit(0);
}

/* Writes from HDF5 file to HDF4 file */
void write_local_SDS_attribute(int32 sds_out, int dim, char * name, int * big_array, 
                               ssize_t ord, ssize_t sign, ssize_t size, ssize_t class)
{
	char charclass[22], charord[22];
	void class_check(ssize_t class, char ret[22]);
	void order_check(ssize_t ord, char ret[22]);
	int32 *i32_array;
	uint32 *ui32_array;
	float64 *f64_array;
	float32 *f32_array;
	int8 *i8_array;
	uint8 *ui8_array;
	int16 *i16_array;
	uint16 *ui16_array;
	long *lng_array;
	unsigned long *ulng_array;
	char *char_array, *tmpchar;
	void swapbytes(void *val,int nbbytes);
	int i;

	switch (class) {
		case H5T_STRING:
			/* DRL: If attribute is string, get the strlen as the number of elements to write */
			printf("H5T_STRING\n");
			printf("Attribute string length = %d\n", strlen((char*)big_array));
			SDsetattr(sds_out, name, DFNT_CHAR, strlen((char*)big_array), (VOIDP)big_array);
			break;
		case H5T_FLOAT:
			printf("H5T_FLOAT\n");
			if (size == 4) {
				f32_array = (float *)big_array;
				if (ord==H5T_ORDER_BE) {
					for(i=0;i<dim;i++) swapbytes(&(f32_array[i]), 4);
				}
				SDsetattr(sds_out, name, DFNT_FLOAT, dim, (VOIDP)f32_array);
			}
			else {
				f64_array = (double *)big_array;
				if (ord==H5T_ORDER_BE) {
					for(i=0;i<dim;i++) swapbytes(&(f64_array[i]), 8);
				}
				SDsetattr(sds_out, name, DFNT_FLOAT64, dim, (VOIDP)f64_array);
			}
			break;
		case H5T_INTEGER:
			printf("H5T_INTEGER\n");
			if (dim == 8) {
				if (sign==H5T_SGN_NONE) {
					ulng_array = (unsigned long *)big_array;
					if (ord==H5T_ORDER_BE) {
					for(i=0;i<dim;i++) swapbytes(&(ulng_array[i]), 8);
					}
					SDsetattr(sds_out, name, DFNT_UINT64, dim, (VOIDP)ulng_array);
				}
				else {
					lng_array = (long *)big_array;
					if (ord==H5T_ORDER_BE) {
						for(i=0;i<dim;i++) swapbytes(&(lng_array[i]), 8);
					}
					SDsetattr(sds_out, name, DFNT_INT64, dim, (VOIDP)ulng_array);
				}
			}
			else if (dim == 4) {
				if (sign==H5T_SGN_NONE) {
					ui32_array = (uint32 *)big_array;
					if (ord==H5T_ORDER_BE) {
						for(i=0;i<dim;i++) swapbytes(&(ui32_array[i]), 4);
					}
					SDsetattr(sds_out, name, DFNT_UINT32, dim, (VOIDP)ui32_array);
				}
				else {
					i32_array = (int32 *)big_array;
					if (ord==H5T_ORDER_BE) {
						for(i=0;i<dim;i++) swapbytes(&(i32_array[i]), 4);
					}
					SDsetattr(sds_out, name, DFNT_INT32, dim, (VOIDP)i32_array);
				}
			}
			else if (dim == 2) {
				if (sign==H5T_SGN_NONE) {
					ui16_array = (uint16 *)big_array;
					if (ord==H5T_ORDER_BE) {
						for(i=0;i<dim;i++) swapbytes(&(ui16_array[i]), 2);
					}
					SDsetattr(sds_out, name, DFNT_UINT16, dim, (VOIDP)ui16_array);
				}
				else {
					i16_array = (int16 *)big_array;
					if (ord==H5T_ORDER_BE) {
						for(i=0;i<dim;i++) swapbytes(&(i16_array[i]), 2);
					}
					SDsetattr(sds_out, name, DFNT_INT16, dim, (VOIDP)i16_array);
				}
			}
			else {
				if (sign==H5T_SGN_NONE) {
					ui8_array = (uint8 *)big_array;
					SDsetattr(sds_out, name, DFNT_UINT8, dim, (VOIDP)ui8_array);
				}
				else {
					i8_array = (int8 *)big_array;
					SDsetattr(sds_out, name, DFNT_INT8, dim, (VOIDP)i8_array);
				}
			}
			break;
		default:
			printf("ERROR: Unknown attribute data type\n");
			break;
	}

	return;
}

/******************************************************************************
!C
!Routine: swapbytes

!Description:  Does a byte reversal on value "val."

!Revision History:
 Original version:    Nazmi Z El Saleous and Eric Vermote

!Input Parameters:
        val           an array of values (cast to void)
        nbbytes       the length of val (in bytes; cannot be greater than 17 
                      bytes)
        
!Output Parameters:
        val           (updated)

!Return value:
        none
                
!References and Credits:

!Developers:
      Nazmi Z El Saleous
      Eric Vermote
      University of Maryland / Dept. of Geography
      nazmi.elsaleous@gsfc.nasa.gov

!Design Notes:  
      
!END
*******************************************************************************/
void swapbytes(void *val, int nbbytes) {

	char *tmpptr1,*tmpptr2,tmpstr[17];
	int i;
   
	memcpy(tmpstr,val,nbbytes);
	tmpptr1=(char *) val;
	tmpptr2=(char *) tmpstr;
	for (i=0;i<nbbytes;i++)
	tmpptr1[i]=tmpptr2[nbbytes-i-1];   
}

void class_check(ssize_t class, char ret[22])
{
	switch(class) {
		case H5T_INTEGER:
			strcpy(ret, "INTEGER\0");
			break;
		case H5T_FLOAT:
			strcpy(ret, "FLOAT\0");
			break;
		case H5T_STRING:
			strcpy(ret, "STRING\0");
			break;
		case H5T_BITFIELD:
			strcpy(ret, "BITFIELD\0");
			break;
		case H5T_OPAQUE:
			strcpy(ret, "OPAQUE\0");
			break;
		case H5T_COMPOUND:
			strcpy(ret, "COMPOUND\0");
			break;
		case H5T_REFERENCE:
			strcpy(ret, "REFERENCE\0");
			break;
		case H5T_ENUM:
			strcpy(ret, "ENUM\0");
			break;
		case H5T_VLEN:
			strcpy(ret, "VLEN\0");
			break;
		case H5T_ARRAY:
			strcpy(ret, "ARRAY\0");
			break;
		default:
			strcpy(ret, "\0");
			break;
	}
	return;
}

void order_check(ssize_t ord, char ret[22])
{
	switch(ord) {
		case H5T_ORDER_LE:
			strcpy(ret, "little-endian\0");
			break;
		case H5T_ORDER_BE:
			strcpy(ret, "big-endian\0");
			break;
		case H5T_ORDER_VAX:
			strcpy(ret, "VAX mixed-byte order\0");
			break;
		/*case H5T_ORDER_MIXED:
			strcpy(ret, "mixed, compound\0");
			break;*/
		case H5T_ORDER_NONE:
			strcpy(ret, "none\0");
			break;
		default:
			strcpy(ret, "\0");
			break;
	}
	return;    
}
