/* 
	Copyright © 1999-2007, United States Government as represented by the 
	Administrator for The National Aeronautics and Space Administration.
	All Rights Reserved.
*/
/*
!C******************************************************************
!Description:
	Functions to handle output to HDF files.

!Revision History:

!References and Credits:
	W. Schroeder, University of Maryland
	W. Walsh, University of Maryland

!Design Notes:

!END*************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <unistd.h>

#include "hdf5.h"
#include "hdf.h"
#include "mfhdf.h"

#include "const.h"
#include "types.h"
#include "input.h"
#include "output.h"
#include "error.h"
#include "misc.h"
#include "version.h"
#include <time.h>



int InitOutputHDF(OutputInfo *, InputInfo *);
extern int write_tb, write_ref;
static int set_dimensions(int32 sds_id, char *dimname1, char *dimname2);
static int write_band_scan(int32 sd_id, int32 sds_index, int32 start[2],
	int32 edges[2], void *buff);
static char *satstr(enum Satellite sat);


/*
* Purpose: Top function for writing data to hdf/ascii files
*
* Inputs:	ScaledVal
*		scale
*		offset
*		scan_lines
*
* Outputs: 	UnscaledVal
*
* Returns 0 for success, otherwise non-zero error code.
*/
int WriteOutput(OutputInfo *out_info, InputInfo *in_info, Juggernaut *jugg, FireData *fire_data, StaticFireReference *txt_fire_ref, StaticFireReference *mem_fire_ref){
	/* Use only for new VNP_QSLMWIP files */
	FlagTrimmedBowtie(jugg);

	/* Writes jugg->pixflag.mask to HDF output */
	WriteMask(out_info, jugg);

	/* Writes static fire reference file*/
	WriteRefFile(in_info, jugg, txt_fire_ref, mem_fire_ref);
	
	/* Populates empty FireData fields from the Juggernaut */
	if( (out_info->write_ascii || out_info->write_hdf) && fire_data->index != 0 ) PopulateFireData(jugg, fire_data);
	
	/* Sort FireData fields */
	if( (out_info->write_ascii || out_info->write_hdf) && fire_data->index != 0 ) SortFireData(fire_data);
	
	/* Write FireData info to hdf SDSs */
	if(out_info->write_hdf) WriteFireDataSDSs(out_info, fire_data);
	
	/* Write FireData info to ascii file */
	if(out_info->write_ascii && fire_data->index != 0) WriteFireDataASCII(out_info, in_info, jugg, fire_data);
	
	/* Write post run global attributes */
	WriteGlobalAttr(out_info, jugg, fire_data);

	/* Close output files */
	if (out_info->write_netcdf == 0){
		if (SDend(out_info->sd_id_fire)) exit(1);
	}else{
		if (H5Fclose(out_info->sd_id_fire)) exit(1);
	}

	if (out_info->write_ascii && fire_data->index != 0) fclose(out_info->ascii_ptr);
	
}
void WriteAttr_NetCDF(hid_t sd_id, hid_t typeid, char *attr_name, void *buff){
	int rt;
	hid_t attr_id, dataspace_id, attr_dim[2];
	attr_dim[0] = 1;
	attr_dim[1] = 0;


	dataspace_id = H5Screate_simple(1, attr_dim, NULL);
	attr_id = H5Acreate1(sd_id, attr_name, typeid, dataspace_id, H5P_DEFAULT);
	rt = H5Awrite(attr_id, typeid, buff);
	
	/* Close the attribute. */
	rt = H5Aclose(attr_id);
	rt = H5Sclose(dataspace_id);

}

int InitOutputNetCDF(OutputInfo *out_info, InputInfo *in_info){
	int rt;
	hsize_t     dims[2];
	hid_t	dataspace_id;
	hid_t       cparms; 
	hid_t       rank; 
	hsize_t      chunk_dims[2];
	hsize_t attr_dim;
	hid_t attr_id;
	hid_t typeid;
	char *legend;

	/**/
	rank = 2;
	dims[0] = in_info->nscans * LINES_PER_SCAN_375M;
   	dims[1] = SAMPLES_375M;
	chunk_dims[0] = LINES_PER_SCAN_375M;
	chunk_dims[1] = SAMPLES_375M;
	attr_dim = 1;

legend =
	"\n0 not-processed (non-zero QF)\n"
	"1 bowtie\n"
	"2 glint\n"
	"3 water\n"
	"4 clouds\n"
	"5 clear land\n"
	"6 unclassified fire pixel\n"
	"7 low confidence fire pixel\n"
	"8 nominal confidence fire pixel\n"
	"9 high confidence fire pixel\n\0";


	/* Open hdf5 output */
	out_info->sd_id_fire = H5Fcreate(out_info->hdf_name, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);

	/**** fire mask ****/
	/* Modify dataset creation properties, i.e. enable chunking  */
	cparms = H5Pcreate(H5P_DATASET_CREATE);
	rt = H5Pset_chunk(cparms, rank, chunk_dims);
	rt = H5Pset_deflate (cparms, 6);

	/* Create SDS */
	dataspace_id = H5Screate_simple(rank, dims, NULL);
   	out_info->sds_index_mask = H5Dcreate1(out_info->sd_id_fire, "fire mask", H5T_NATIVE_UINT8, dataspace_id, cparms);

	/* legend attr */
	typeid =  H5Tcopy(H5T_C_S1);
	rt = H5Tset_size(typeid, strlen(legend));
	WriteAttr_NetCDF(out_info->sds_index_mask, typeid, "legend", legend);

	rt = H5Sclose(dataspace_id);
	rt = H5Dclose(out_info->sds_index_mask);
	/****  ****/

	/**** algorithm QA ****/
	/* Modify dataset creation properties, i.e. enable chunking  */
	cparms = H5Pcreate(H5P_DATASET_CREATE);
	rt = H5Pset_chunk(cparms, rank, chunk_dims);
	rt = H5Pset_deflate (cparms, 6);

	/* Create SDS */
	dataspace_id = H5Screate_simple(rank, dims, NULL);
   	out_info->sds_index_qa = H5Dcreate1(out_info->sd_id_fire, "algorithm QA", H5T_NATIVE_UINT32, dataspace_id, cparms);

	/* units attr */
	typeid =  H5Tcopy(H5T_C_S1);
	rt = H5Tset_size(typeid, strlen("bit field\0"));
	WriteAttr_NetCDF(out_info->sds_index_qa, typeid, "units", "bit field\0");

	rt = H5Sclose(dataspace_id);
	rt = H5Dclose(out_info->sds_index_qa);
	/****  ****/


	/**** Set global attr ****/
	typeid =  H5Tcopy(H5T_C_S1);
	rt = H5Tset_size(typeid, strlen(in_info->viae_filename));
	WriteAttr_NetCDF(out_info->sd_id_fire, typeid, "VIAES", in_info->viae_filename);

	typeid =  H5Tcopy(H5T_C_S1);
	rt = H5Tset_size(typeid, strlen(in_info->imft_filename));
	WriteAttr_NetCDF(out_info->sd_id_fire, typeid, "IMFTS", in_info->imft_filename);

	typeid =  H5Tcopy(H5T_C_S1);
	rt = H5Tset_size(typeid, strlen(in_info->vmae_filename));
	WriteAttr_NetCDF(out_info->sd_id_fire, typeid, "VMAES", in_info->vmae_filename);

	typeid =  H5Tcopy(H5T_C_S1);
	rt = H5Tset_size(typeid, strlen(in_info->vcdgip_filename));
	WriteAttr_NetCDF(out_info->sd_id_fire, typeid, "VCDGIPS", in_info->vcdgip_filename);

	typeid =  H5Tcopy(H5T_C_S1);
	rt = H5Tset_size(typeid, strlen(in_info->qslwmip_filename));
	WriteAttr_NetCDF(out_info->sd_id_fire, typeid, "QSLWMIP", in_info->qslwmip_filename);

	typeid =  H5Tcopy(H5T_C_S1);
	rt = H5Tset_size(typeid, strlen(PROCESS_VERSION_NUMBER));
	WriteAttr_NetCDF(out_info->sd_id_fire, typeid, "ProcessVersionNumber", PROCESS_VERSION_NUMBER);
	
	typeid =  H5Tcopy(H5T_C_S1);
	rt = H5Tset_size(typeid, strlen(__DATE__));
	WriteAttr_NetCDF(out_info->sd_id_fire, typeid, "ExecutableCreationDate", __DATE__);

	typeid =  H5Tcopy(H5T_C_S1);
	rt = H5Tset_size(typeid, strlen(__TIME__));
	WriteAttr_NetCDF(out_info->sd_id_fire, typeid, "ExecutableCreationTime", __TIME__);

	typeid =  H5Tcopy(H5T_C_S1);
	rt = H5Tset_size(typeid, strlen("NPP\0"));
	WriteAttr_NetCDF(out_info->sd_id_fire, typeid, "Satellite", "NPP\0");

	typeid =  H5Tcopy(H5T_C_S1);
	rt = H5Tset_size(typeid, strlen(make_uname()));
	WriteAttr_NetCDF(out_info->sd_id_fire, typeid, "SystemID", make_uname());


	/********/
	char gringlat[2000];
	char gringlon[2000];
	

	attr_id = SDfindattr(in_info->sd_id_imft, "Unagg_GRingLatitude");
	rt = SDreadattr(in_info->sd_id_imft, attr_id, gringlat);

	typeid =  H5Tcopy(H5T_C_S1);
	rt = H5Tset_size(typeid, strlen(gringlat));
	WriteAttr_NetCDF(out_info->sd_id_fire, typeid, "Unagg_GRingLatitude", gringlat);

	attr_id = SDfindattr(in_info->sd_id_imft, "Unagg_GRingLongitude");
	rt = SDreadattr(in_info->sd_id_imft, attr_id, gringlon);

	typeid =  H5Tcopy(H5T_C_S1);
	rt = H5Tset_size(typeid, strlen(gringlon));
	WriteAttr_NetCDF(out_info->sd_id_fire, typeid, "Unagg_GRingLongitude", gringlon);
	/********/
	

	/********/
	float temp_float[1];

	attr_id = SDfindattr(in_info->sd_id_imft, "NorthBoundingCoord");
	rt = SDreadattr(in_info->sd_id_imft, attr_id, temp_float);

	WriteAttr_NetCDF(out_info->sd_id_fire, H5T_NATIVE_FLOAT, "NorthBoundingCoordinate", temp_float);

	attr_id = SDfindattr(in_info->sd_id_imft, "SouthBoundingCoord");
	rt = SDreadattr(in_info->sd_id_imft, attr_id, temp_float);

	WriteAttr_NetCDF(out_info->sd_id_fire, H5T_NATIVE_FLOAT, "SouthBoundingCoordinate", temp_float);
	
	attr_id = SDfindattr(in_info->sd_id_imft, "EastBoundingCoord");
	rt = SDreadattr(in_info->sd_id_imft, attr_id, temp_float);

	WriteAttr_NetCDF(out_info->sd_id_fire, H5T_NATIVE_FLOAT, "EastBoundingCoordinate", temp_float);

	attr_id = SDfindattr(in_info->sd_id_imft, "WestBoundingCoord");
	rt = SDreadattr(in_info->sd_id_imft, attr_id, temp_float);

	WriteAttr_NetCDF(out_info->sd_id_fire, H5T_NATIVE_FLOAT, "WestBoundingCoordinate", temp_float);
	/********/

	/* StartTime */
	char buff_str[50];
	attr_id = SDfindattr(in_info->sd_id_viae, "StartTime");
	rt = SDreadattr(in_info->sd_id_viae, attr_id, buff_str);

	typeid =  H5Tcopy(H5T_C_S1);
	rt = H5Tset_size(typeid, strlen(buff_str));
	WriteAttr_NetCDF(out_info->sd_id_fire, typeid, "StartTime", buff_str);
	
	/* EndTime */
	attr_id = SDfindattr(in_info->sd_id_viae, "EndTime");
	rt = SDreadattr(in_info->sd_id_viae, attr_id, buff_str);

	typeid =  H5Tcopy(H5T_C_S1);
	rt = H5Tset_size(typeid, strlen(buff_str));
	WriteAttr_NetCDF(out_info->sd_id_fire, typeid, "EndTime", buff_str);



	/**** ****/
	

	return 0;
}

/* Sets up and fills in the OutputInfo struct */
/* Sets chuncking */
/* Writes a few global attributes*/
int InitOutputHDF(OutputInfo *out_info, InputInfo *in_info){
	
	
	char msg[MAX_MSG_LEN];
	int32 sds_id;
	int rt;

	int32 dim_sizes[2];
	char *dimname1, *dimname2;

	int sds_index = 0;

	int32 n_values;
	
	char units_str[1000];
	char *units;
	units = &units_str;
	char *legend;

	HDF_CHUNK_DEF cdef;

	/* Create output file id */
	if ((out_info->sd_id_fire = SDstart(out_info->hdf_name, DFACC_CREATE)) == FAIL) {
		printf("Unable to open HDF output file \"%s\".\n", out_info->hdf_name);
		exit(1);
	}
	
	dim_sizes[0] = in_info->nscans * LINES_PER_SCAN_375M;
	dim_sizes[1] = SAMPLES_375M;

	dimname1 = "number_of_scan_lines";
	dimname2 = "pixels_per_scan_line";

	cdef.chunk_lengths[0] = cdef.comp.chunk_lengths[0] = LINES_PER_SCAN_375M;
	cdef.chunk_lengths[1] = cdef.comp.chunk_lengths[1] = SAMPLES_375M;
	cdef.comp.comp_type = COMP_CODE_DEFLATE;
	cdef.comp.cinfo.deflate.level = 4;

	/* ----- Global Attr ----- */

	n_values = strlen(in_info->viae_filename);
	if (SDsetattr(out_info->sd_id_fire, "VIAES", DFNT_CHAR8, n_values, in_info->viae_filename))
		return error(ERR_HDF);

	n_values = strlen(in_info->imft_filename);
	if (SDsetattr(out_info->sd_id_fire, "IMFTS", DFNT_CHAR8, n_values, in_info->imft_filename))
		return error(ERR_HDF);

	n_values = strlen(in_info->vmae_filename);
	if (SDsetattr(out_info->sd_id_fire, "VMAES", DFNT_CHAR8, n_values, in_info->vmae_filename))
		return error(ERR_HDF);

	n_values = strlen(in_info->vcdgip_filename);
	if (SDsetattr(out_info->sd_id_fire, "VCDGIPS", DFNT_CHAR8, n_values, in_info->vcdgip_filename))
		return error(ERR_HDF);

	n_values = strlen(in_info->qslwmip_filename);
	if (SDsetattr(out_info->sd_id_fire, "QSLWMIP", DFNT_CHAR8, n_values, in_info->qslwmip_filename))
		return error(ERR_HDF);
	

	char *temp_ptr = PROCESS_VERSION_NUMBER;
	n_values = strlen(temp_ptr);
	if (SDsetattr(out_info->sd_id_fire, "ProcessVersionNumber", DFNT_CHAR8, n_values, temp_ptr))
		return error(ERR_HDF);
	
	temp_ptr = __DATE__;
	n_values = strlen(temp_ptr);
	if (SDsetattr(out_info->sd_id_fire, "ExecutableCreationDate", DFNT_CHAR8, n_values, temp_ptr))
		return error(ERR_HDF);

	temp_ptr = __TIME__;
	n_values = strlen(temp_ptr);
	if (SDsetattr(out_info->sd_id_fire, "ExecutableCreationTime", DFNT_CHAR8, n_values, temp_ptr))
		return error(ERR_HDF);

	char Sat[4];
	strcpy(Sat, "NPP");
	temp_ptr = &Sat;
	n_values = strlen(temp_ptr);
	if (SDsetattr(out_info->sd_id_fire, "Satellite", DFNT_CHAR8, n_values, temp_ptr))
		return error(ERR_HDF);

	char *s;
	s = make_uname();
	n_values = strlen(s);
	if (SDsetattr(out_info->sd_id_fire, "SystemID", DFNT_CHAR8, n_values, s))
		return error(ERR_HDF);

	/***** GRing Lat/Lon & Bounding Coords *****/
	/* GRing Lat/Lon */
	char sds_str[100];
	char num_str[3];
	int Granules;
	int *Granules_ptr;
	Granules_ptr = &Granules;
	char gringlat[2000];
	char gringlon[2000];
	char temp_str[300];
	float32 *tempf_ptr;
	float32 buff_f[1];
	int i;
	float array[8];
	float *array_ptr;
	array_ptr = &array;
	int32 attr_id;
	
	/* GRingLat */
	attr_id = SDfindattr(in_info->sd_id_imft, "Unagg_GRingLatitude");
	rt = SDreadattr(in_info->sd_id_imft, attr_id, gringlat);
	temp_ptr = &gringlat;
	n_values = strlen(temp_ptr);
	if (SDsetattr(out_info->sd_id_fire, "Unagg_GRingLatitude", DFNT_CHAR8, n_values, temp_ptr))
		return error(ERR_HDF);
	
	/* GRingLon */
	attr_id = SDfindattr(in_info->sd_id_imft, "Unagg_GRingLongitude");
	rt = SDreadattr(in_info->sd_id_imft, attr_id, gringlon);
	temp_ptr = &gringlon;
	n_values = strlen(temp_ptr);
	if (SDsetattr(out_info->sd_id_fire, "Unagg_GRingLongitude", DFNT_CHAR8, n_values, temp_ptr))
		return error(ERR_HDF);
	
	/* North Bounding */
	attr_id = SDfindattr(in_info->sd_id_imft, "NorthBoundingCoord");
	rt = SDreadattr(in_info->sd_id_imft, attr_id, buff_f);
	tempf_ptr = &buff_f[0];
	if (SDsetattr(out_info->sd_id_fire, "NorthBoundingCoordinate", DFNT_FLOAT, 1, tempf_ptr))
		return error(ERR_HDF);

	/* South Bounding */
	attr_id = SDfindattr(in_info->sd_id_imft, "SouthBoundingCoord");
	rt = SDreadattr(in_info->sd_id_imft, attr_id, buff_f);
	tempf_ptr = &buff_f[0];
	if (SDsetattr(out_info->sd_id_fire, "SouthBoundingCoordinate", DFNT_FLOAT, 1, tempf_ptr))
		return error(ERR_HDF);

	/* East Bounding */
	attr_id = SDfindattr(in_info->sd_id_imft, "EastBoundingCoord");
	rt = SDreadattr(in_info->sd_id_imft, attr_id, buff_f);
	tempf_ptr = &buff_f[0];
	if (SDsetattr(out_info->sd_id_fire, "EastBoundingCoordinate", DFNT_FLOAT, 1, tempf_ptr))
		return error(ERR_HDF);

	/* West Bounding */
	attr_id = SDfindattr(in_info->sd_id_imft, "WestBoundingCoord");
	rt = SDreadattr(in_info->sd_id_imft, attr_id, buff_f);
	tempf_ptr = &buff_f[0];
	if (SDsetattr(out_info->sd_id_fire, "WestBoundingCoordinate", DFNT_FLOAT, 1, tempf_ptr))
		return error(ERR_HDF);
	/****			****/

	/* StartTime */
	char buff_str[50];
	attr_id = SDfindattr(in_info->sd_id_viae, "StartTime");
	rt = SDreadattr(in_info->sd_id_viae, attr_id, buff_str);
	if (SDsetattr(out_info->sd_id_fire, "StartTime", DFNT_CHAR8, strlen(&buff_str[0]), &buff_str[0]))
		return error(ERR_HDF);

	/* EndTime */
	attr_id = SDfindattr(in_info->sd_id_viae, "EndTime");
	rt = SDreadattr(in_info->sd_id_viae, attr_id, buff_str);
	if (SDsetattr(out_info->sd_id_fire, "EndTime", DFNT_CHAR8, strlen(&buff_str[0]), &buff_str[0]))
		return error(ERR_HDF);

	/* ----- fire mask ----- */

	out_info->sds_index_mask = sds_index++;
	if ((sds_id = SDcreate(out_info->sd_id_fire, "fire mask", DFNT_UINT8, 2,
		dim_sizes)) == FAIL) return error(ERR_HDF);

	if (SDsetchunk(sds_id, cdef, HDF_CHUNK | HDF_COMP) == FAIL)
		return error(ERR_HDF);

	if (set_dimensions(sds_id, dimname1, dimname2))
		return error(ERR_HDF);
legend =
	"\n0 not-processed (non-zero QF)\n"
	"1 bowtie\n"
	"2 glint\n"
	"3 water\n"
	"4 clouds\n"
	"5 clear land\n"
	"6 unclassified fire pixel\n"
	"7 low confidence fire pixel\n"
	"8 nominal confidence fire pixel\n"
	"9 high confidence fire pixel\n";

	/* Add attributes */
	n_values = strlen(legend);
	if (SDsetattr(sds_id, "legend", DFNT_CHAR8, n_values, legend))
		return error(ERR_HDF);

	if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);

	/* ----- QA ----- */

	out_info->sds_index_qa = sds_index++;
	if ((sds_id = SDcreate(out_info->sd_id_fire, "algorithm QA", DFNT_UINT32, 2,
		dim_sizes)) == FAIL)
		return error(ERR_HDF);

	if (SDsetchunk(sds_id, cdef, HDF_CHUNK | HDF_COMP) == FAIL)
		return error(ERR_HDF);

	if (set_dimensions(sds_id, dimname1, dimname2))
		return error(ERR_HDF);

	/* Add attributes */
	strcpy(units, "bit field");
	n_values = strlen(units);
	if (SDsetattr(sds_id, "units", DFNT_CHAR8, n_values, units))
		return error(ERR_HDF);

	if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);

	return 0;
}

void WriteRefFile(InputInfo *in_info, Juggernaut *jugg, StaticFireReference *txt_fire_ref, StaticFireReference *mem_fire_ref){
	int i;
	long delta_t = 30*86400*pow(10,6);
	long time_threshold;

	// DRL: Cast MidTime as long first?
	//time_threshold = jugg->MidTime[(jugg->number_of_scan_lines-1)/32]-delta_t;
	time_threshold = ((long)jugg->MidTime[(jugg->number_of_scan_lines-1)/32]) - delta_t;

	for(i=0;i<txt_fire_ref->entries;i++){
		if (txt_fire_ref->MidTimes[i][3] != 0){
			fprintf(in_info->ref_file_ptr, "  %.3lf, %.3lf, %ld, %ld, %ld\n", txt_fire_ref->Grid_XY[i][0], txt_fire_ref->Grid_XY[i][1], txt_fire_ref->MidTimes[i][1], txt_fire_ref->MidTimes[i][2], txt_fire_ref->MidTimes[i][3]);
		}else{
			if (txt_fire_ref->MidTimes[i][2] > time_threshold)
				fprintf(in_info->ref_file_ptr, "  %.3lf, %.3lf, %ld, %ld, %ld\n", txt_fire_ref->Grid_XY[i][0], txt_fire_ref->Grid_XY[i][1], txt_fire_ref->MidTimes[i][0], txt_fire_ref->MidTimes[i][1], txt_fire_ref->MidTimes[i][2]);
		}
	
	}
	


	
	for(i=0;i<mem_fire_ref->entries;i++){
		if (mem_fire_ref->MidTimes[i][3] != 0){
			fprintf(in_info->ref_file_ptr, "  %.3lf, %.3lf, %ld, %ld, %ld\n", mem_fire_ref->Grid_XY[i][0], mem_fire_ref->Grid_XY[i][1], mem_fire_ref->MidTimes[i][1], mem_fire_ref->MidTimes[i][2], mem_fire_ref->MidTimes[i][3]);
		}
	}
	
	
	/* Flush, unlock and close */
	fflush(in_info->ref_file_ptr);
	if (flock(fileno(in_info->ref_file_ptr), LOCK_UN) != 0){
		printf("Error: Could not unlock ref. file\n");
	}
	if (fclose(in_info->ref_file_ptr) != 0){
		printf("Error: Could not close ref. file\n");
	}

	
	/* Debugging copy 
	refPtr = fopen("/data/wwalsh/ref/ref_w_copy.txt", "w");
	
	for(i=0;i<txt_fire_ref->entries;i++){
		if (txt_fire_ref->MidTimes[i][3] != 0){
			fprintf(refPtr, "  %.3lf, %.3lf, %ld, %ld, %ld\n", txt_fire_ref->Grid_XY[i][0], txt_fire_ref->Grid_XY[i][1], txt_fire_ref->MidTimes[i][1], txt_fire_ref->MidTimes[i][2], txt_fire_ref->MidTimes[i][3]);
		}else{
			fprintf(refPtr, "  %.3lf, %.3lf, %ld, %ld, %ld\n", txt_fire_ref->Grid_XY[i][0], txt_fire_ref->Grid_XY[i][1], txt_fire_ref->MidTimes[i][0], txt_fire_ref->MidTimes[i][1], txt_fire_ref->MidTimes[i][2]);
		}
		
	}

	for(i=0;i<mem_fire_ref->entries;i++){
		if (mem_fire_ref->MidTimes[i][3] != 0){
			fprintf(refPtr, "  %.3lf, %.3lf, %ld, %ld, %ld\n", mem_fire_ref->Grid_XY[i][0], mem_fire_ref->Grid_XY[i][1], mem_fire_ref->MidTimes[i][1], mem_fire_ref->MidTimes[i][2], mem_fire_ref->MidTimes[i][3]);
		}else{
			fprintf(refPtr, "  %.3lf, %.3lf, %ld, %ld, %ld\n", mem_fire_ref->Grid_XY[i][0], mem_fire_ref->Grid_XY[i][1], mem_fire_ref->MidTimes[i][0], mem_fire_ref->MidTimes[i][1], mem_fire_ref->MidTimes[i][2]);
		}
		
	}
	fclose(refPtr);*/
}


void SortFireData(FireData *fire_data){
	uint8 Mask; 
	float Lon;
	float Lat;
	uint16 Row;
	uint16 Col;
	float T4;
	float T5;
	uint16 Window_Size;
	float SolZenAng;
	float SolAzAng;
	float ViewZenAng;
	float ViewAzAng;
	uint8 Day;
	float mean_T4;
	float mean_T5;
	float MAD_T4;
	float MAD_T5;
	float mean_Dif45;
	float MAD_Dif45;
	uint16 Adj_Cloud;
	uint16 Adj_Water;
	float FRP;
	float Rad13;
	float mean_Rad13;



	int x, y;
	int n = fire_data->index;
	int change_made_flag = 0;
	for(x=0; x<n; x++){
		change_made_flag = 0;
		for(y=0; y<n-1; y++){
			if(fire_data->Row[y] > fire_data->Row[y+1]){
				change_made_flag = 1;
				/* Save [y+1] */
				Mask = fire_data->Mask[y+1];
				Lon = fire_data->Lon[y+1];
				Lat = fire_data->Lat[y+1];
				Row = fire_data->Row[y+1];
				Col = fire_data->Col[y+1];
				T4 = fire_data->T4[y+1];
				T5 = fire_data->T5[y+1];
				Window_Size = fire_data->Window_Size[y+1];
				SolZenAng = fire_data->SolZenAng[y+1];
				SolAzAng = fire_data->SolAzAng[y+1];
				ViewZenAng = fire_data->ViewZenAng[y+1];
				ViewAzAng = fire_data->ViewAzAng[y+1];
				Day = fire_data->Day[y+1];
				mean_T4 = fire_data->mean_T4[y+1];
				mean_T5 = fire_data->mean_T5[y+1];
				MAD_T4 = fire_data->MAD_T4[y+1];
				MAD_T5 = fire_data->MAD_T5[y+1];
				mean_Dif45 = fire_data->mean_Dif45[y+1];
				MAD_Dif45 = fire_data->MAD_Dif45[y+1];
				Adj_Cloud = fire_data->Adj_Cloud[y+1];
				Adj_Water = fire_data->Adj_Water[y+1];
				FRP = fire_data->FRP[y+1];
				Rad13 = fire_data->Rad13[y+1];
				mean_Rad13 = fire_data->mean_Rad13[y+1];
				

				/* [y+1] = [y]*/
				fire_data->Mask[y+1] = fire_data->Mask[y];
				fire_data->Lon[y+1] = fire_data->Lon[y];
				fire_data->Lat[y+1] = fire_data->Lat[y];
				fire_data->Row[y+1] = fire_data->Row[y];
				fire_data->Col[y+1] = fire_data->Col[y];
				fire_data->T4[y+1] = fire_data->T4[y];
				fire_data->T5[y+1] = fire_data->T5[y];
				fire_data->Window_Size[y+1] = fire_data->Window_Size[y];
				fire_data->SolZenAng[y+1] = fire_data->SolZenAng[y];
				fire_data->SolAzAng[y+1] = fire_data->SolAzAng[y];
				fire_data->ViewZenAng[y+1] = fire_data->ViewZenAng[y];
				fire_data->ViewAzAng[y+1] = fire_data->ViewAzAng[y];
				fire_data->Day[y+1] = fire_data->Day[y];
				fire_data->mean_T4[y+1] = fire_data->mean_T4[y];
				fire_data->mean_T5[y+1] = fire_data->mean_T5[y];
				fire_data->MAD_T4[y+1] = fire_data->MAD_T4[y];
				fire_data->MAD_T5[y+1] = fire_data->MAD_T5[y];
				fire_data->mean_Dif45[y+1] = fire_data->mean_Dif45[y];
				fire_data->MAD_Dif45[y+1] = fire_data->MAD_Dif45[y];
				fire_data->Adj_Cloud[y+1] = fire_data->Adj_Cloud[y];
				fire_data->Adj_Water[y+1] = fire_data->Adj_Water[y];
				fire_data->FRP[y+1]= fire_data->FRP[y];
				fire_data->Rad13[y+1] = fire_data->Rad13[y];
				fire_data->mean_Rad13[y+1] = fire_data->mean_Rad13[y];
				

				/* [y] = [y+1]*/
				fire_data->Mask[y] = Mask;
				fire_data->Lon[y] = Lon;
				fire_data->Lat[y] = Lat;
				fire_data->Row[y] = Row;
				fire_data->Col[y] = Col;
				fire_data->T4[y] = T4;
				fire_data->T5[y] = T5;
				fire_data->Window_Size[y] = Window_Size;
				fire_data->SolZenAng[y] = SolZenAng;
				fire_data->SolAzAng[y] = SolAzAng;
				fire_data->ViewZenAng[y] = ViewZenAng;
				fire_data->ViewAzAng[y] = ViewAzAng;
				fire_data->Day[y] = Day;
				fire_data->mean_T4[y] = mean_T4;
				fire_data->mean_T5[y] = mean_T5;
				fire_data->MAD_T4[y] = MAD_T4;
				fire_data->MAD_T5[y] = MAD_T5;
				fire_data->mean_Dif45[y] = mean_Dif45;
				fire_data->MAD_Dif45[y] = MAD_Dif45;
				fire_data->Adj_Cloud[y] = Adj_Cloud;
				fire_data->Adj_Water[y] = Adj_Water;
				fire_data->FRP[y]= FRP;
				fire_data->Rad13[y] = Rad13;
				fire_data->mean_Rad13[y] = mean_Rad13;
				
			}
		}
		if(change_made_flag == 0) break;
	}

}

/* Write FireData info to ascii file */
void WriteFireDataASCII(OutputInfo *out_info, InputInfo *in_info, Juggernaut *jugg, FireData *fire_data){
	char scan_str[40];
	int scan_num;
	char Scan_Time[20];
	int f;
	char Confidence[8];
	char Period[6];

	double fire_secs;
	time_t fire_secs_since_epoch;
	time_t scan_t;
	struct tm *scan_ptr;
	double scan0_secs;
	double beginning_secs;

	/* Get beginning time in seconds since 1970 */
	char year[5];
	char month[3];
	char day[3];
	char hour[3];
	char minute[3];
	char second[3];

	strncpy(&year, &in_info->BeginningDate[0], 4);
	strncpy(&month, &in_info->BeginningDate[4], 2);
	strncpy(&day, &in_info->BeginningDate[6], 2);
	strncpy(&hour, &in_info->BeginningTime[0], 2);
	strncpy(&minute, &in_info->BeginningTime[2], 2);
	strncpy(&second, &in_info->BeginningTime[4], 2);
	year[4] = '\0';
	month[2] = '\0';
	day[2] = '\0';
	hour[2] = '\0';
	minute[2] = '\0';
	second[2] = '\0';

	struct tm Beginning;
	Beginning.tm_year = atoi(year) - 1900;
	Beginning.tm_mon = atoi(month) - 1;
	Beginning.tm_mday = atoi(day);
	Beginning.tm_hour = atoi(hour);
	Beginning.tm_min = atoi(minute);
	Beginning.tm_sec = atoi(second);
	Beginning.tm_isdst = 0;

	putenv("TZ=UTC");
	beginning_secs = mktime(&Beginning);
	/*******************/
	
	/* midtime[0] seconds since 1957 */
	scan0_secs = jugg->MidTime[0]/1000000;

	/* Writes header in ascii file */
	InitOutputASCII(out_info, in_info, fire_data);

	for(f=0;f<fire_data->index;f++){
		if(fire_data->Mask[f] == 8 || fire_data->Mask[f] == 9){
			strcpy(Confidence, "nominal");
		}else if(fire_data->Mask[f] == 7){
			strcpy(Confidence, "low");
		}
		
		if(fire_data->Day[f] == 1){
			strcpy(Period, "day");
		}else{
			strcpy(Period, "night");
		}
		
		/* Scan Time */
		scan_num = fire_data->Row[f] / 32;
		fire_secs = jugg->MidTime[scan_num]/1000000;
		fire_secs_since_epoch = beginning_secs + (fire_secs - scan0_secs);
		scan_ptr = gmtime(&fire_secs_since_epoch);	
		strftime(scan_str, sizeof(scan_str), "%Y%m%d_%H:%M:%S\0",scan_ptr);
		strcpy(Scan_Time, scan_str);


		fprintf(out_info->ascii_ptr, "%13s,%18s,%11.5f,%10.5f,%7d,%7d,%9.1f,%9.1f,%11s,%7s,%7.2f\n", &fire_data->Granule, &Scan_Time, fire_data->Lon[f], fire_data->Lat[f], fire_data->Col[f], fire_data->Row[f], fire_data->T4[f], fire_data->T5[f], &Confidence, &Period, fire_data->FRP[f]);
	}
	fflush(out_info->ascii_ptr);
}

/* Write FireData info to hdf SDSs */
/* (could shorten by creating SDS writing module) */
int CreateAndWriteSDS(int sd_id, char *sdsName, int dataType, int entries, void *data){
	int32 sds_id;
	int32 dim_id;
	int32 dim_sizes[2];
	int start[1];
	int edges[1];
	
	dim_sizes[0] = entries;
	char *dimname1;
	dimname1 = "number_of_fires";
	start[0] = 0;
	edges[0] = dim_sizes[0];

	if ((sds_id = SDcreate(sd_id, sdsName, dataType, 1, dim_sizes)) == FAIL)
		exit(1);

	if (dim_sizes[0] != 0){
		if ((dim_id = SDgetdimid(sds_id, 0)) == FAIL) return ERR_HDF;
		if (SDsetdimname(dim_id, dimname1) == FAIL) return ERR_HDF;

		/* Write */	
		if (SDwritedata(sds_id, start, NULL, edges, data) == FAIL)
			exit(1);
	}
	return sds_id;
}

/* */
hid_t CreateAndWriteSDS_NetCDF(hid_t sd_id, char *sdsName, hid_t dataType, int entries, void *data){
	hid_t sds_id, dataspace_id;
	int rt, rank, dims[2];
	hid_t cparms; 

	rank = 1;
	dims[0] = entries;
	dims[1] = 0;

	cparms = H5Pcreate(H5P_DATASET_CREATE);
	rt = H5Pset_deflate (cparms, 6);

	/* Create SDS */
	dataspace_id = H5Screate_simple(rank, dims, NULL);
   	sds_id = H5Dcreate1(sd_id, sdsName, dataType, dataspace_id, H5P_DEFAULT);
	rt = H5Sclose(dataspace_id);

	/* Write SDS */
	rt = H5Dwrite(sds_id, dataType, H5S_ALL, H5S_ALL, H5P_DEFAULT, data);

	
	return sds_id;
	
}

void WriteSDSAttr(int sds_id, char *attrName, char *attrValue){
	int32 n_values;

	n_values = strlen(attrValue);
	if (SDsetattr(sds_id, attrName, DFNT_CHAR8, n_values, attrValue))
		exit(1);

}

void WriteFireDataSDSs(OutputInfo *out_info, FireData *fire_data){
	int32 rt;
	int sds_index = out_info->sds_index_qa;
	char *buff;
	hid_t typeid, dataspace_id, sds_id;
	int rank, dims[2];
	rank = 1;
	dims[0] = fire_data->index;
	
	int test[1];
	test[0] = 11;
	
	/* --- Row --- */
	if (out_info->write_netcdf == 0){
		out_info->sds_index_row = sds_index++;
		sds_id = CreateAndWriteSDS(out_info->sd_id_fire, "FP_line", DFNT_UINT16, fire_data->index, fire_data->Row);
		WriteSDSAttr(sds_id, "long_name", "granule line of fire pixel");
		if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	}else{		
		sds_id = CreateAndWriteSDS_NetCDF(out_info->sd_id_fire, "FP_line", H5T_NATIVE_UINT16,  fire_data->index, fire_data->Row);

		typeid =  H5Tcopy(H5T_C_S1);
		rt = H5Tset_size(typeid, strlen("granule line of fire pixel\0"));
		WriteAttr_NetCDF(sds_id, typeid, "long_name", "granule line of fire pixel\0");
		rt = H5Dclose(sds_id);	
	}



	/* --- Col --- */
	if (out_info->write_netcdf == 0){
		out_info->sds_index_col = sds_index++;
		sds_id = CreateAndWriteSDS(out_info->sd_id_fire, "FP_sample", DFNT_UINT16, fire_data->index, fire_data->Col);
		WriteSDSAttr(sds_id, "long_name", "granule sample of fire pixel");
		if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	}else{
		sds_id = CreateAndWriteSDS_NetCDF(out_info->sd_id_fire, "FP_sample", H5T_NATIVE_UINT16,  fire_data->index, fire_data->Col);

		typeid =  H5Tcopy(H5T_C_S1);
		rt = H5Tset_size(typeid, strlen("granule sample of fire pixel\0"));
		WriteAttr_NetCDF(sds_id, typeid, "long_name", "granule sample of fire pixel\0");
		
		rt = H5Dclose(sds_id);
	}

	/* --- Lat --- */
	if (out_info->write_netcdf == 0){
		out_info->sds_index_lat = sds_index++;
		sds_id = CreateAndWriteSDS(out_info->sd_id_fire, "FP_latitude", DFNT_FLOAT32, fire_data->index, fire_data->Lat);
		WriteSDSAttr(sds_id, "long_name", "latitude of fire pixel");
		WriteSDSAttr(sds_id, "units", "degrees");
		if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	}else{
		sds_id = CreateAndWriteSDS_NetCDF(out_info->sd_id_fire, "FP_latitude", H5T_NATIVE_FLOAT,  fire_data->index, fire_data->Lat);

		typeid =  H5Tcopy(H5T_C_S1);
		rt = H5Tset_size(typeid, strlen("latitude of fire pixel\0"));
		WriteAttr_NetCDF(sds_id, typeid, "long_name", "latitude of fire pixel\0");
		
		rt = H5Dclose(sds_id);
	}

	/* --- Lon --- */
	if (out_info->write_netcdf == 0){
		out_info->sds_index_lon = sds_index++;
		sds_id = CreateAndWriteSDS(out_info->sd_id_fire, "FP_longitude", DFNT_FLOAT32, fire_data->index, fire_data->Lon);
		WriteSDSAttr(sds_id, "long_name", "longitude of fire pixel");
		WriteSDSAttr(sds_id, "units", "degrees");
		if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	}else{
		sds_id = CreateAndWriteSDS_NetCDF(out_info->sd_id_fire, "FP_longitude", H5T_NATIVE_FLOAT,  fire_data->index, fire_data->Lon);

		typeid =  H5Tcopy(H5T_C_S1);
		rt = H5Tset_size(typeid, strlen("longitude of fire pixel\0"));
		WriteAttr_NetCDF(sds_id, typeid, "long_name", "longitude of fire pixel\0");
		
		rt = H5Dclose(sds_id);
	}

	/* --- BT4 --- */
	if (out_info->write_netcdf == 0){
		out_info->sds_index_T4 = sds_index++;
		sds_id = CreateAndWriteSDS(out_info->sd_id_fire, "FP_T4", DFNT_FLOAT32, fire_data->index, fire_data->T4);
		WriteSDSAttr(sds_id, "long_name", "I04 brightness temperature of fire pixel");
		WriteSDSAttr(sds_id, "units", "kelvins");
		if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	}else{
		sds_id = CreateAndWriteSDS_NetCDF(out_info->sd_id_fire, "FP_T4", H5T_NATIVE_FLOAT,  fire_data->index, fire_data->T4);

		typeid =  H5Tcopy(H5T_C_S1);
		rt = H5Tset_size(typeid, strlen("I04 brightness temperature of fire pixel\0"));
		WriteAttr_NetCDF(sds_id, typeid, "long_name", "I04 brightness temperature of fire pixel\0");

		typeid =  H5Tcopy(H5T_C_S1);
		rt = H5Tset_size(typeid, strlen("kelvins\0"));
		WriteAttr_NetCDF(sds_id, typeid, "units", "kelvins\0");
		
		rt = H5Dclose(sds_id);
	}

	/* --- BT5 --- */
	if (out_info->write_netcdf == 0){
		out_info->sds_index_T5 = sds_index++;
		sds_id = CreateAndWriteSDS(out_info->sd_id_fire, "FP_T5", DFNT_FLOAT32, fire_data->index, fire_data->T5);
		WriteSDSAttr(sds_id, "long_name", "I05 brightness temperature of fire pixel");
		WriteSDSAttr(sds_id, "units", "kelvins");
		if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	}else{
		sds_id = CreateAndWriteSDS_NetCDF(out_info->sd_id_fire, "FP_T5", H5T_NATIVE_FLOAT,  fire_data->index, fire_data->T5);

		typeid =  H5Tcopy(H5T_C_S1);
		rt = H5Tset_size(typeid, strlen("I05 brightness temperature of fire pixel\0"));
		WriteAttr_NetCDF(sds_id, typeid, "long_name", "I05 brightness temperature of fire pixel\0");

		typeid =  H5Tcopy(H5T_C_S1);
		rt = H5Tset_size(typeid, strlen("kelvins\0"));
		WriteAttr_NetCDF(sds_id, typeid, "units", "kelvins\0");
		
		rt = H5Dclose(sds_id);
	}

	/* --- mean_T4 --- */
	if (out_info->write_netcdf == 0){
		out_info->sds_index_MeanT4 = sds_index++;
		sds_id = CreateAndWriteSDS(out_info->sd_id_fire, "FP_MeanT4", DFNT_FLOAT32, fire_data->index, fire_data->mean_T4);
		WriteSDSAttr(sds_id, "long_name", "I04 brightness temperature of background");
		WriteSDSAttr(sds_id, "units", "kelvins");
		if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	}else{
		sds_id = CreateAndWriteSDS_NetCDF(out_info->sd_id_fire, "FP_MeanT4", H5T_NATIVE_FLOAT,  fire_data->index, fire_data->mean_T4);

		typeid =  H5Tcopy(H5T_C_S1);
		rt = H5Tset_size(typeid, strlen("I04 brightness temperature of background\0"));
		WriteAttr_NetCDF(sds_id, typeid, "long_name", "I04 brightness temperature of background\0");
		
		typeid =  H5Tcopy(H5T_C_S1);
		rt = H5Tset_size(typeid, strlen("kelvins\0"));
		WriteAttr_NetCDF(sds_id, typeid, "units", "kelvins\0");

		rt = H5Dclose(sds_id);
	}

	/* --- mean_T5 --- */
	if (out_info->write_netcdf == 0){
		out_info->sds_index_MeanT5 = sds_index++;
		sds_id = CreateAndWriteSDS(out_info->sd_id_fire, "FP_MeanT5", DFNT_FLOAT32, fire_data->index, fire_data->mean_T5);
		WriteSDSAttr(sds_id, "long_name", "I05 brightness temperature of background");
		WriteSDSAttr(sds_id, "units", "kelvins");
		if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	}else{
		sds_id = CreateAndWriteSDS_NetCDF(out_info->sd_id_fire, "FP_MeanT5", H5T_NATIVE_FLOAT,  fire_data->index, fire_data->mean_T5);

		typeid =  H5Tcopy(H5T_C_S1);
		rt = H5Tset_size(typeid, strlen("I05 brightness temperature of background\0"));
		WriteAttr_NetCDF(sds_id, typeid, "long_name", "I05 brightness temperature of background\0");

		typeid =  H5Tcopy(H5T_C_S1);
		rt = H5Tset_size(typeid, strlen("kelvins\0"));
		WriteAttr_NetCDF(sds_id, typeid, "units", "kelvins\0");
		
		rt = H5Dclose(sds_id);
	}

	/* --- mean_Dif45 --- */
	if (out_info->write_netcdf == 0){
		out_info->sds_index_MeanDif = sds_index++;
		sds_id = CreateAndWriteSDS(out_info->sd_id_fire, "FP_MeanDT", DFNT_FLOAT32, fire_data->index, fire_data->mean_Dif45);
		WriteSDSAttr(sds_id, "long_name", "mean background brightness temperature difference");
		WriteSDSAttr(sds_id, "units", "kelvins");
		if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	}else{
		sds_id = CreateAndWriteSDS_NetCDF(out_info->sd_id_fire, "FP_MeanDT", H5T_NATIVE_FLOAT,  fire_data->index, fire_data->mean_Dif45);

		typeid =  H5Tcopy(H5T_C_S1);
		rt = H5Tset_size(typeid, strlen("mean background brightness temperature difference\0"));
		WriteAttr_NetCDF(sds_id, typeid, "long_name", "mean background brightness temperature difference\0");

		typeid =  H5Tcopy(H5T_C_S1);
		rt = H5Tset_size(typeid, strlen("kelvins\0"));
		WriteAttr_NetCDF(sds_id, typeid, "units", "kelvins\0");
		
		rt = H5Dclose(sds_id);
	}

	/* --- MAD_T4 --- */
	if (out_info->write_netcdf == 0){
		out_info->sds_index_MADT4 = sds_index++;
		sds_id = CreateAndWriteSDS(out_info->sd_id_fire, "FP_MAD_T4", DFNT_FLOAT32, fire_data->index, fire_data->MAD_T4);
		WriteSDSAttr(sds_id, "long_name", "background I04 brightness temperature mean absolute deviation");
		WriteSDSAttr(sds_id, "units", "kelvins");
		if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	}else{
		sds_id = CreateAndWriteSDS_NetCDF(out_info->sd_id_fire, "FP_MAD_T4", H5T_NATIVE_FLOAT,  fire_data->index, fire_data->MAD_T4);

		typeid =  H5Tcopy(H5T_C_S1);
		rt = H5Tset_size(typeid, strlen("background I04 brightness temperature mean absolute deviation\0"));
		WriteAttr_NetCDF(sds_id, typeid, "long_name", "background I04 brightness temperature mean absolute deviation\0");

		typeid =  H5Tcopy(H5T_C_S1);
		rt = H5Tset_size(typeid, strlen("kelvins\0"));
		WriteAttr_NetCDF(sds_id, typeid, "units", "kelvins\0");
		
		rt = H5Dclose(sds_id);
	}

	/* --- MAD_T5 --- */
	if (out_info->write_netcdf == 0){
		out_info->sds_index_MADT5 = sds_index++;
		sds_id = CreateAndWriteSDS(out_info->sd_id_fire, "FP_MAD_T5", DFNT_FLOAT32, fire_data->index, fire_data->MAD_T5);
		WriteSDSAttr(sds_id, "long_name", "background I05 brightness temperature mean absolute deviation");
		WriteSDSAttr(sds_id, "units", "kelvins");
		if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	}else{
		sds_id = CreateAndWriteSDS_NetCDF(out_info->sd_id_fire, "FP_MAD_T5", H5T_NATIVE_FLOAT,  fire_data->index, fire_data->MAD_T5);

		typeid =  H5Tcopy(H5T_C_S1);
		rt = H5Tset_size(typeid, strlen("background I05 brightness temperature mean absolute deviation\0"));
		WriteAttr_NetCDF(sds_id, typeid, "long_name", "background I05 brightness temperature mean absolute deviation\0");

		typeid =  H5Tcopy(H5T_C_S1);
		rt = H5Tset_size(typeid, strlen("kelvins\0"));
		WriteAttr_NetCDF(sds_id, typeid, "units", "kelvins\0");
		
		rt = H5Dclose(sds_id);
	}

	/* --- MADDT --- */
	if (out_info->write_netcdf == 0){
		out_info->sds_index_MADDif = sds_index++;
		sds_id = CreateAndWriteSDS(out_info->sd_id_fire, "FP_MAD_DT", DFNT_FLOAT32, fire_data->index, fire_data->MAD_Dif45);
		WriteSDSAttr(sds_id, "long_name", "background brightness temperature difference mean absolute deviation");
		WriteSDSAttr(sds_id, "units", "kelvins");
		if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	}else{
		sds_id = CreateAndWriteSDS_NetCDF(out_info->sd_id_fire, "FP_MAD_DT", H5T_NATIVE_FLOAT,  fire_data->index, fire_data->MAD_Dif45);

		typeid =  H5Tcopy(H5T_C_S1);
		rt = H5Tset_size(typeid, strlen("background brightness temperature difference mean absolute deviation\0"));
		WriteAttr_NetCDF(sds_id, typeid, "long_name", "background brightness temperature difference mean absolute deviation\0");

		typeid =  H5Tcopy(H5T_C_S1);
		rt = H5Tset_size(typeid, strlen("kelvins\0"));
		WriteAttr_NetCDF(sds_id, typeid, "units", "kelvins\0");
		
		rt = H5Dclose(sds_id);
	}

	/* --- FRP --- */
	if (out_info->write_netcdf == 0){
		out_info->sds_index_FRP = sds_index++;
		sds_id = CreateAndWriteSDS(out_info->sd_id_fire, "FP_power", DFNT_FLOAT32, fire_data->index, fire_data->FRP);
		WriteSDSAttr(sds_id, "long_name", "fire radiative power");
		WriteSDSAttr(sds_id, "units", "MW");
		if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	}else{
		sds_id = CreateAndWriteSDS_NetCDF(out_info->sd_id_fire, "FP_power", H5T_NATIVE_FLOAT,  fire_data->index, fire_data->FRP);

		typeid =  H5Tcopy(H5T_C_S1);
		rt = H5Tset_size(typeid, strlen("fire radiative power\0"));
		WriteAttr_NetCDF(sds_id, typeid, "long_name", "fire radiative power\0");

		typeid =  H5Tcopy(H5T_C_S1);
		rt = H5Tset_size(typeid, strlen("MW\0"));
		WriteAttr_NetCDF(sds_id, typeid, "units", "MW\0");
		
		rt = H5Dclose(sds_id);
	}

	/* --- Rad13 --- */
	if (out_info->write_netcdf == 0){
		out_info->sds_index_Rad13 = sds_index++;
		sds_id = CreateAndWriteSDS(out_info->sd_id_fire, "FP_Rad13", DFNT_FLOAT32, fire_data->index, fire_data->Rad13);
		WriteSDSAttr(sds_id, "long_name", "M13 radiance of fire pixel");
		WriteSDSAttr(sds_id, "units", "W/(m^2*sr*μm)");
		if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	}else{
		sds_id = CreateAndWriteSDS_NetCDF(out_info->sd_id_fire, "FP_Rad13", H5T_NATIVE_FLOAT,  fire_data->index, fire_data->Rad13);

		typeid =  H5Tcopy(H5T_C_S1);
		rt = H5Tset_size(typeid, strlen("M13 radiance of fire pixel\0"));
		WriteAttr_NetCDF(sds_id, typeid, "long_name", "M13 radiance of fire pixel\0");

		typeid =  H5Tcopy(H5T_C_S1);
		rt = H5Tset_size(typeid, strlen("W/(m^2*sr*μm)\0"));
		WriteAttr_NetCDF(sds_id, typeid, "units", "W/(m^2*sr*μm)\0");
		
		rt = H5Dclose(sds_id);
	}

	/* --- mean_Rad13 --- */
	if (out_info->write_netcdf == 0){
		out_info->sds_index_MeanRad13 = sds_index++;
		sds_id = CreateAndWriteSDS(out_info->sd_id_fire, "FP_MeanRad13", DFNT_FLOAT32, fire_data->index, fire_data->mean_Rad13);
		WriteSDSAttr(sds_id, "long_name", "M13 background radiance of background");
		WriteSDSAttr(sds_id, "units", "W/(m^2*sr*μm)");
		if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	}else{
		sds_id = CreateAndWriteSDS_NetCDF(out_info->sd_id_fire, "FP_MeanRad13", H5T_NATIVE_FLOAT,  fire_data->index, fire_data->mean_Rad13);

		typeid =  H5Tcopy(H5T_C_S1);
		rt = H5Tset_size(typeid, strlen("M13 background radiance of background\0"));
		WriteAttr_NetCDF(sds_id, typeid, "long_name", "M13 background radiance of background\0");

		typeid =  H5Tcopy(H5T_C_S1);
		rt = H5Tset_size(typeid, strlen("W/(m^2*sr*μm)\0"));
		WriteAttr_NetCDF(sds_id, typeid, "units", "W/(m^2*sr*μm)\0");
		
		rt = H5Dclose(sds_id);
	}

	/* --- Adj Cloud --- */
	if (out_info->write_netcdf == 0){
		out_info->sds_index_AdjCloud = sds_index++;
		sds_id = CreateAndWriteSDS(out_info->sd_id_fire, "FP_AdjCloud", DFNT_UINT16, fire_data->index, fire_data->Adj_Cloud);
		WriteSDSAttr(sds_id, "long_name", "number of adjacent cloud pixels");
		if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	}else{
		sds_id = CreateAndWriteSDS_NetCDF(out_info->sd_id_fire, "FP_AdjCloud", H5T_NATIVE_UINT16,  fire_data->index, fire_data->Adj_Cloud);

		typeid =  H5Tcopy(H5T_C_S1);
		rt = H5Tset_size(typeid, strlen("number of adjacent cloud pixels\0"));
		WriteAttr_NetCDF(sds_id, typeid, "long_name", "number of adjacent cloud pixels\0");
		
		rt = H5Dclose(sds_id);
	}

	/* --- Adj Water --- */
	if (out_info->write_netcdf == 0){
		out_info->sds_index_AdjWater = sds_index++;
		sds_id = CreateAndWriteSDS(out_info->sd_id_fire, "FP_AdjWater", DFNT_UINT16, fire_data->index, fire_data->Adj_Water);
		WriteSDSAttr(sds_id, "long_name", "number of adjacent water pixels");
		if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	}else{
		sds_id = CreateAndWriteSDS_NetCDF(out_info->sd_id_fire, "FP_AdjWater", H5T_NATIVE_UINT16,  fire_data->index, fire_data->Adj_Water);

		typeid =  H5Tcopy(H5T_C_S1);
		rt = H5Tset_size(typeid, strlen("number of adjacent water pixels\0"));
		WriteAttr_NetCDF(sds_id, typeid, "long_name", "number of adjacent water pixels\0");
		
		rt = H5Dclose(sds_id);
	}

	/* --- Window_Size --- */
	if (out_info->write_netcdf == 0){
		out_info->sds_index_WindowSize = sds_index++;
		sds_id = CreateAndWriteSDS(out_info->sd_id_fire, "FP_WinSize", DFNT_UINT16, fire_data->index, fire_data->Window_Size);
		WriteSDSAttr(sds_id, "long_name", "background window size");
		if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	}else{
		sds_id = CreateAndWriteSDS_NetCDF(out_info->sd_id_fire, "FP_WinSize", H5T_NATIVE_UINT16,  fire_data->index, fire_data->Window_Size);

		typeid =  H5Tcopy(H5T_C_S1);
		rt = H5Tset_size(typeid, strlen("background window size\0"));
		WriteAttr_NetCDF(sds_id, typeid, "long_name", "background window size\0");
		
		rt = H5Dclose(sds_id);
	}

	/* --- Confidence --- */
	if (out_info->write_netcdf == 0){
		out_info->sds_index_Confidence = sds_index++;
		sds_id = CreateAndWriteSDS(out_info->sd_id_fire, "FP_confidence", DFNT_UINT8, fire_data->index, fire_data->Mask);
		WriteSDSAttr(sds_id, "long_name", "detection confidence");
		if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	}else{
		sds_id = CreateAndWriteSDS_NetCDF(out_info->sd_id_fire, "FP_confidence", H5T_NATIVE_UINT8,  fire_data->index, fire_data->Mask);

		typeid =  H5Tcopy(H5T_C_S1);
		rt = H5Tset_size(typeid, strlen("detection confidence\0"));
		WriteAttr_NetCDF(sds_id, typeid, "long_name", "detection confidence\0");
		
		rt = H5Dclose(sds_id);
	}

	/* --- Day --- */
	if (out_info->write_netcdf == 0){
		out_info->sds_index_day = sds_index++;
		sds_id = CreateAndWriteSDS(out_info->sd_id_fire, "FP_day", DFNT_UINT8, fire_data->index, fire_data->Day);
		WriteSDSAttr(sds_id, "long_name", "day flag for fire pixel");
		if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	}else{
		sds_id = CreateAndWriteSDS_NetCDF(out_info->sd_id_fire, "FP_day", H5T_NATIVE_UINT8,  fire_data->index, fire_data->Day);

		typeid =  H5Tcopy(H5T_C_S1);
		rt = H5Tset_size(typeid, strlen("day flag for fire pixel\0"));
		WriteAttr_NetCDF(sds_id, typeid, "long_name", "day flag for fire pixel\0");
		
		rt = H5Dclose(sds_id);
	}

	/* --- SolZenAng --- */
	if (out_info->write_netcdf == 0){
		out_info->sds_index_SolZenAng = sds_index++;
		sds_id = CreateAndWriteSDS(out_info->sd_id_fire, "FP_SolZenAng", DFNT_FLOAT32, fire_data->index, fire_data->SolZenAng);
		WriteSDSAttr(sds_id, "long_name", "solar zenith angle of fire pixel");
		WriteSDSAttr(sds_id, "units", "degrees");
		if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	}else{
		sds_id = CreateAndWriteSDS_NetCDF(out_info->sd_id_fire, "FP_SolZenAng", H5T_NATIVE_FLOAT,  fire_data->index, fire_data->SolZenAng);

		typeid =  H5Tcopy(H5T_C_S1);
		rt = H5Tset_size(typeid, strlen("solar zenith angle of fire pixel\0"));
		WriteAttr_NetCDF(sds_id, typeid, "long_name", "solar zenith angle of fire pixel\0");

		typeid =  H5Tcopy(H5T_C_S1);
		rt = H5Tset_size(typeid, strlen("degrees\0"));
		WriteAttr_NetCDF(sds_id, typeid, "units", "degrees\0");
		
		rt = H5Dclose(sds_id);
	}

	/* --- SolAzAng --- */
	if (out_info->write_netcdf == 0){
		out_info->sds_index_SolAzAng = sds_index++;
		sds_id = CreateAndWriteSDS(out_info->sd_id_fire, "FP_SolAzAng", DFNT_FLOAT32, fire_data->index, fire_data->SolAzAng);
		WriteSDSAttr(sds_id, "long_name", "solar azimuth angle of fire pixel");
		WriteSDSAttr(sds_id, "units", "degrees");
		if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	}else{
		sds_id = CreateAndWriteSDS_NetCDF(out_info->sd_id_fire, "FP_SolAzAng", H5T_NATIVE_FLOAT,  fire_data->index, fire_data->SolAzAng);

		typeid =  H5Tcopy(H5T_C_S1);
		rt = H5Tset_size(typeid, strlen("solar azimuth angle of fire pixel\0"));
		WriteAttr_NetCDF(sds_id, typeid, "long_name", "solar azimuth angle of fire pixel\0");

		typeid =  H5Tcopy(H5T_C_S1);
		rt = H5Tset_size(typeid, strlen("degrees\0"));
		WriteAttr_NetCDF(sds_id, typeid, "units", "degrees\0");
		
		rt = H5Dclose(sds_id);
	}

	/* --- ViewZenAng --- */
	if (out_info->write_netcdf == 0){
		out_info->sds_index_ViewZenAng = sds_index++;
		sds_id = CreateAndWriteSDS(out_info->sd_id_fire, "FP_ViewZenAng", DFNT_FLOAT32, fire_data->index, fire_data->ViewZenAng);
		WriteSDSAttr(sds_id, "long_name", "view zenith angle of fire pixel");
		WriteSDSAttr(sds_id, "units", "degrees");
		if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	}else{
		sds_id = CreateAndWriteSDS_NetCDF(out_info->sd_id_fire, "FP_ViewZenAng", H5T_NATIVE_FLOAT,  fire_data->index, fire_data->ViewZenAng);

		typeid =  H5Tcopy(H5T_C_S1);
		rt = H5Tset_size(typeid, strlen("view zenith angle of fire pixel\0"));
		WriteAttr_NetCDF(sds_id, typeid, "long_name", "view zenith angle of fire pixel\0");

		typeid =  H5Tcopy(H5T_C_S1);
		rt = H5Tset_size(typeid, strlen("degrees\0"));
		WriteAttr_NetCDF(sds_id, typeid, "units", "degrees\0");
		
		rt = H5Dclose(sds_id);
	}

	/* --- ViewAzAng --- */
	if (out_info->write_netcdf == 0){
		out_info->sds_index_ViewAzAng = sds_index++;
		sds_id = CreateAndWriteSDS(out_info->sd_id_fire, "FP_ViewAzAng", DFNT_FLOAT32, fire_data->index, fire_data->ViewAzAng);
		WriteSDSAttr(sds_id, "long_name", "view azimuth angle of fire pixel");
		WriteSDSAttr(sds_id, "units", "degrees");
		if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	}else{
		sds_id = CreateAndWriteSDS_NetCDF(out_info->sd_id_fire, "FP_ViewAzAng", H5T_NATIVE_FLOAT,  fire_data->index, fire_data->ViewAzAng);

		typeid =  H5Tcopy(H5T_C_S1);
		rt = H5Tset_size(typeid, strlen("view azimuth angle of fire pixel\0"));
		WriteAttr_NetCDF(sds_id, typeid, "long_name", "granule line of fire pixel\0");
		
		typeid =  H5Tcopy(H5T_C_S1);
		rt = H5Tset_size(typeid, strlen("degrees\0"));
		WriteAttr_NetCDF(sds_id, typeid, "units", "degrees\0");

		rt = H5Dclose(sds_id);
	}


	return 0;
}

/* Writes jugg->pixflag.mask to HDF output */
int WriteMask(OutputInfo *out_info, Juggernaut *jugg){
	int i, j, rt;
	int32 sds_id;
	int32 start[2];
	int32 edges[2];

	uint8 **mask;
	if ( !(mask = (uint8**)malloc(jugg->number_of_scan_lines*sizeof(uint8*))) )
		return error(ERR_F_MEM);
	if ( !(mask[0] = (uint8*)malloc(jugg->number_of_scan_lines*SAMPLES_375M*sizeof(uint8))) )
		return error(ERR_F_MEM);
	for (i=1;i<jugg->number_of_scan_lines;i++){
		mask[i] = mask[i-1] + SAMPLES_375M;
	}

	uint32 **QA;
	if ( !(QA = (uint32**)malloc(jugg->number_of_scan_lines*sizeof(uint32*))) )
		return error(ERR_F_MEM);
	if ( !(QA[0] = (uint32*)malloc(jugg->number_of_scan_lines*SAMPLES_375M*sizeof(uint32))) )
		return error(ERR_F_MEM);
	for (i=1;i<jugg->number_of_scan_lines;i++){
		QA[i] = QA[i-1] + SAMPLES_375M;
	}


	for(i=0;i<jugg->number_of_scan_lines;i++){
		for(j=0;j<SAMPLES_375M;j++){
			mask[i][j] = jugg->pixflags[i][j].mask;
			QA[i][j] = jugg->pixflags[i][j].meta_data;
		}
	}
	if (out_info->write_netcdf == 0){
		start[0] = 0;
		edges[0] = jugg->number_of_scan_lines;

		start[1] = 0;
		edges[1] = SAMPLES_375M;

		/* Write fire mask */
		if ((sds_id = SDselect(out_info->sd_id_fire, out_info->sds_index_mask)) == FAIL)
			return ERR_HDF;
		if (SDwritedata(sds_id, start, NULL, edges, mask[0]) == FAIL)
			return ERR_HDF;
		if (SDendaccess(sds_id) == FAIL) return ERR_HDF;

		/* Write algorithm QA */
		if ((sds_id = SDselect(out_info->sd_id_fire, out_info->sds_index_qa)) == FAIL)
			return ERR_HDF;
		if (SDwritedata(sds_id, start, NULL, edges, QA[0]) == FAIL)
			return ERR_HDF;
		if (SDendaccess(sds_id) == FAIL) return ERR_HDF;
	}else{
		//hid_t file = H5Fopen(FILE, H5F_ACC_WRONLY, H5P_DEFAULT);
    		hid_t dataset = H5Dopen(out_info->sd_id_fire, "fire mask",H5P_DEFAULT);
		rt = H5Dwrite(dataset, H5T_NATIVE_UINT8, H5S_ALL, H5S_ALL, H5P_DEFAULT, mask[0]);

    		dataset = H5Dopen(out_info->sd_id_fire, "algorithm QA",H5P_DEFAULT);
		rt = H5Dwrite(dataset, H5T_NATIVE_UINT32, H5S_ALL, H5S_ALL, H5P_DEFAULT, QA[0]);

		//H5Fclose(out_info->sd_id_fire);
		//H5Dwrite( hid_t dataset_id, hid_t mem_type_id, hid_t mem_space_id, hid_t file_space_id, hid_t xfer_plist_id, const void * buf ) 
	}
	
	free(mask[0]);
	free(mask);
	free(QA[0]);
	free(QA);
}

/* Fills in other FireData elements */
void PopulateFireData(Juggernaut *jugg, FireData *fire_data){
	int mask;
	int i, j, f, x, y;
	int cloud_count;
	int water_count;
	int redact_count = 0;

	for(f=0;f<fire_data->index;f++){
		i = fire_data->Row[f];
		j = fire_data->Col[f];

		mask = jugg->pixflags[i][j].mask;

		/* Fire was later rejected */
		if(mask != 7 && mask != 8 && mask != 9){
			redact_count++;
			continue;
		}

		fire_data->Mask[f-redact_count] = mask;
		fire_data->Row[f-redact_count] = i;
		fire_data->Col[f-redact_count] = j;
		fire_data->mean_T4[f-redact_count] = fire_data->mean_T4[f];
		fire_data->mean_T5[f-redact_count] = fire_data->mean_T5[f];
		fire_data->MAD_T4[f-redact_count] = fire_data->MAD_T4[f];
		fire_data->MAD_T5[f-redact_count] = fire_data->MAD_T5[f];
		fire_data->mean_Dif45[f-redact_count] = fire_data->mean_Dif45[f];
		fire_data->MAD_Dif45[f-redact_count] = fire_data->MAD_Dif45[f];
		fire_data->Window_Size[f-redact_count] = fire_data->Window_Size[f];
		fire_data->Lon[f-redact_count] = jugg->lon[i][j];
		fire_data->Lat[f-redact_count] = jugg->lat[i][j];
		fire_data->T4[f-redact_count] = jugg->T4[i][j];
		fire_data->T5[f-redact_count] = jugg->T5[i][j];
		fire_data->SolZenAng[f-redact_count] = jugg->SolZenAng[i][j];
		fire_data->SolAzAng[f-redact_count] = jugg->SolAzAng[i][j];
		fire_data->ViewZenAng[f-redact_count] = jugg->ViewZenAng[i][j];
		fire_data->ViewAzAng[f-redact_count] = jugg->ViewAzAng[i][j];
		fire_data->Day[f-redact_count] = jugg->pixflags[i][j].day;
		fire_data->FRP[f-redact_count] = fire_data->FRP[f];
		fire_data->Rad13[f-redact_count] = fire_data->Rad13[f];
		fire_data->mean_Rad13[f-redact_count] = fire_data->mean_Rad13[f];

		/* Count adj water and clouds */
		cloud_count = 0;
		water_count = 0;

		for(y=-1;y<=1;y++){
			for(x=-1;x<=1;x++){
				if(y == 0 && x == 0) continue;
				if(i+y < 0 || i+y >= jugg->number_of_scan_lines) continue;
				if(j+x < 0 || j+x >= SAMPLES_375M) continue;


				if(jugg->pixflags[i+y][j+x].water_mask == 1){
					water_count++;
				}
				if(jugg->pixflags[i+y][j+x].mask == 4){
					cloud_count++;
				}

			}
		}			
		
		fire_data->Adj_Cloud[f-redact_count] = cloud_count;
		fire_data->Adj_Water[f-redact_count] = water_count;	
	}
	fire_data->index = fire_data->index - redact_count;
	
}

/* Write post run global attributes */
void WriteGlobalAttr(OutputInfo *out_info, Juggernaut *jugg, FireData *fire_data){
	int day_night_both_flag = 0; // day=1,night=2,both=3
	char day_night_both_str[10];
	int n_values;
	int32 *temp_ptr;
	hid_t typeid;

	/* Counts */
	int total_day_pixels = jugg->day_count;
	int total_night_pixels = jugg->night_count;
	int total_missing_pixels = jugg->missing_count;
	int total_glint_pixels = jugg->glint_count;
	int total_land_pixels = jugg->land_count;
	int total_water_pixels = jugg->water_count;
	int total_cloud_pixels = jugg->cloud_count;
	//int total_glintRejection_pixels = jugg->glint_rejection_count;
	if (out_info->write_netcdf == 0){
		if(total_night_pixels == 0){
			strcpy(day_night_both_str, "Day");
		}else if(total_day_pixels == 0){
			strcpy(day_night_both_str, "Night");
		}else{
			strcpy(day_night_both_str, "Both");
		}

		n_values = strlen(day_night_both_str);
		if (SDsetattr(out_info->sd_id_fire, "Day/Night/Both", DFNT_CHAR8, n_values, day_night_both_str))
			exit(1);

		temp_ptr = &fire_data->index;
		if (SDsetattr(out_info->sd_id_fire, "FirePix", DFNT_INT32, 1, temp_ptr))
			exit(1);
	
		temp_ptr = &total_day_pixels;
		if (SDsetattr(out_info->sd_id_fire, "DayPix", DFNT_INT32, 1, temp_ptr))
			exit(1);

		temp_ptr = &total_night_pixels;
		if (SDsetattr(out_info->sd_id_fire, "NightPix", DFNT_INT32, 1, temp_ptr))
			exit(1);

		temp_ptr = &total_land_pixels;
		if (SDsetattr(out_info->sd_id_fire, "LandPix", DFNT_INT32, 1, temp_ptr))
			exit(1);

		temp_ptr = &total_water_pixels;
		if (SDsetattr(out_info->sd_id_fire, "WaterPix", DFNT_INT32, 1, temp_ptr))
			exit(1);

		temp_ptr = &total_missing_pixels;
		if (SDsetattr(out_info->sd_id_fire, "MissingPix", DFNT_INT32, 1, temp_ptr))
			exit(1);

		temp_ptr = &total_glint_pixels;
		if (SDsetattr(out_info->sd_id_fire, "GlintPix", DFNT_INT32, 1, temp_ptr))
			exit(1);

		temp_ptr = &total_cloud_pixels;
		if (SDsetattr(out_info->sd_id_fire, "CloudPix", DFNT_INT32, 1, temp_ptr))
			exit(1);

		/*temp_ptr = &total_glintRejection_pixels;
		if (SDsetattr(out_info->sd_id_fire, "GlintRejectedPix", DFNT_INT32, 1, temp_ptr))
			exit(1);*/
	}else{
		
		if(total_night_pixels == 0){
			strcpy(day_night_both_str, "Day");
		}else if(total_day_pixels == 0){
			strcpy(day_night_both_str, "Night");
		}else{
			strcpy(day_night_both_str, "Both");
		}

		typeid =  H5Tcopy(H5T_C_S1);
		H5Tset_size(typeid, strlen(day_night_both_str));
		WriteAttr_NetCDF(out_info->sd_id_fire, typeid, "Day/Night/Both", day_night_both_str);
		
		WriteAttr_NetCDF(out_info->sd_id_fire, H5T_NATIVE_INT32, "FirePix", &fire_data->index);
		
		WriteAttr_NetCDF(out_info->sd_id_fire, H5T_NATIVE_INT32, "DayPix", &total_day_pixels);

		WriteAttr_NetCDF(out_info->sd_id_fire, H5T_NATIVE_INT32, "NightPix", &total_night_pixels);

		WriteAttr_NetCDF(out_info->sd_id_fire, H5T_NATIVE_INT32, "LandPix", &total_land_pixels);

		WriteAttr_NetCDF(out_info->sd_id_fire, H5T_NATIVE_INT32, "WaterPix", &total_water_pixels);

		WriteAttr_NetCDF(out_info->sd_id_fire, H5T_NATIVE_INT32, "MissingPix", &total_missing_pixels);

		WriteAttr_NetCDF(out_info->sd_id_fire, H5T_NATIVE_INT32, "GlintPix", &total_glint_pixels);

		WriteAttr_NetCDF(out_info->sd_id_fire, H5T_NATIVE_INT32, "CloudPix", &total_cloud_pixels);

	}

}

/* Writes the header for the ascii file*/
void InitOutputASCII(OutputInfo *out_info, InputInfo *in_info, FireData *fire_data){
	int len, date_pos;
	int offset = 34;

	/* Header */
	char VIIRS_Granule[20];
	strcpy(VIIRS_Granule, "VIIRS_Granule");
	char Scan_Time[12];
	strcpy(Scan_Time, "Scan_Time");
	char Lon[4];
	strcpy(Lon, "Lon");
	char Lat[4];
	strcpy(Lat, "Lat");
	char Sample[7];
	strcpy(Sample, "Sample");
	char Line[5];
	strcpy(Line, "Line");
	char BT_I4[9];
	strcpy(BT_I4, "BT_I4(K)");
	char BT_I5[9];
	strcpy(BT_I5, "BT_I5(K)");
	char Confidence[15];
	strcpy(Confidence, "Confidence");
	char Period[7];
	strcpy(Period, "Period");
	char FRP[4];
	strcpy(FRP, "FRP");

	/* Getting the granule time */
	len = strlen(in_info->viae_filename);
	date_pos = len - offset;
	strncpy(fire_data->Granule, in_info->viae_filename + date_pos, 12);
	fire_data->Granule[12] = '\0';

	/* Opening file and writing first line */ 
	out_info->ascii_ptr = fopen(out_info->ascii_name, "w");

	fprintf(out_info->ascii_ptr, "%13s,%18s,%11s,%10s,%7s,%7s,%9s,%9s,%11s,%7s,%7s\n", &VIIRS_Granule, &Scan_Time, &Lon, &Lat, &Sample, &Line, &BT_I4, &BT_I5, &Confidence, &Period, &FRP);
	fflush(out_info->ascii_ptr);

	return 0;
}

/* Sets the dimension names */
static int set_dimensions(int32 sds_id, char *dimname1, char *dimname2){
	int32 dim_id;

	if ((dim_id = SDgetdimid(sds_id, 0)) == FAIL) return ERR_HDF;
	if (SDsetdimname(dim_id, dimname1) == FAIL) return ERR_HDF;
	if ((dim_id = SDgetdimid(sds_id, 1)) == FAIL) return ERR_HDF;
	if (SDsetdimname(dim_id, dimname2) == FAIL) return ERR_HDF;

	return 0;
}

void FlagTrimmedBowtie(Juggernaut *jugg){
	int LEFT_BOWTIE_BOUND = 2180;
	int RIGHT_BOWTIE_BOUND = 4220;
	int sample_750m, scan_line, i, j;

	for(i=0;i<jugg->number_of_scan_lines;i++){
		for(j=0;j<LEFT_BOWTIE_BOUND;j++){
			if (jugg->pixflags[i][j].mask >= 7 && jugg->pixflags[i][j].mask <= 9){
				scan_line = (i/2)%16;
				sample_750m = j/2;

				switch(scan_line) {

				case 0: case 15:
					if (sample_750m >= 1008 && sample_750m < 1090)
						jugg->pixflags[i][j].meta_data |= 0x400000; /* Meta data 21 */	
					break;
				case 1: case 14:
					if (sample_750m >= 640 && sample_750m < 820)
						jugg->pixflags[i][j].meta_data |= 0x400000; /* Meta data 21 */	
					break;
				case 2: case 13:
					if (sample_750m >= 0 && sample_750m < 520)
						jugg->pixflags[i][j].meta_data |= 0x400000; /* Meta data 21 */	
					break;
				case 3: case 12:
					if (sample_750m >= 0 && sample_750m < 130)
						jugg->pixflags[i][j].meta_data |= 0x400000; /* Meta data 21 */	
					break;

				default :
				continue;
				}


			}
		}
	}

	for(i=0;i<jugg->number_of_scan_lines;i++){
		for(j=RIGHT_BOWTIE_BOUND;j<SAMPLES_375M;j++){
			if (jugg->pixflags[i][j].mask >= 7 && jugg->pixflags[i][j].mask <= 9){
				scan_line = (i/2)%16;
				sample_750m = j/2;

				switch(scan_line) {

				case 0: case 15:
					if (sample_750m >= 2110 && sample_750m < 2192)
						jugg->pixflags[i][j].meta_data |= 0x400000; /* Meta data 21 */	
					break;
				case 1: case 14:
					if (sample_750m >= 2380 && sample_750m < 2560)
						jugg->pixflags[i][j].meta_data |= 0x400000; /* Meta data 21 */	
					break;
				case 2: case 13:
					if (sample_750m >= SAMPLES_750M-520 && sample_750m < SAMPLES_750M)
						jugg->pixflags[i][j].meta_data |= 0x400000; /* Meta data 21 */	
					break;
				case 3: case 12:
					if (sample_750m >= SAMPLES_750M-130 && sample_750m < SAMPLES_750M)
						jugg->pixflags[i][j].meta_data |= 0x400000; /* Meta data 21 */	
					break;

				default :
				continue;
				}


			}
		}
	}




}



