/* 
	Copyright © 1999-2007, United States Government as represented by the 
	Administrator for The National Aeronautics and Space Administration.
	All Rights Reserved.
*/
/*
!C******************************************************************
!Description:

	Functions related to processing input.

!Revision History:
!Team-unique Header:
!References and Credits:

	W. Schroeder, University of Maryland
	W. Walsh, University of Maryland

	Last Edit: February 2016

!END*************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "hdf5.h"
#include <time.h>

#include "const.h"
#include "types.h"

#include "output.h"
#include "input.h"
#include "process.h"
#include "frp.h"

#include "error.h"




/* ---------- global flags ---------- */
extern int verbose;
extern int write_coarse;
extern int write_cmg;


/* This is the top-level function */
int ProcessInput(InputInfo *in_info, OutputInfo *out_info)
{ 	
	int i, rt;

	/* Allocate ALL input arrays */
	/* Also contains information on each pixel (day, glint, missing, etc.) */
	Juggernaut *jugg;
	if ( !(jugg = (Juggernaut *) malloc(sizeof(Juggernaut))) )
		return error(ERR_F_MEM);

	/* Initalize Juggernaut (create function) */
	jugg->number_of_scan_lines = in_info->nscans * LINES_PER_SCAN_375M;
	jugg->day_count = 0;
	jugg->night_count = 0;
	jugg->land_count = 0;
	jugg->water_count = 0;
	jugg->missing_count = 0;
	jugg->glint_count = 0;
	jugg->cloud_count = 0;
	jugg->glint_rejection_count = 0;
	jugg->sa_night_pass = 0;

	/* Allocate FireData arrays */
	/* FireData will hold information on each fire found */
	FireData *fire_data;
	if ( !(fire_data = (FireData *) malloc(sizeof(FireData))) )
		return error(ERR_F_MEM);

	/* Set all ptrs to NULL */
	FireDataInit(fire_data);

	/* Realloc the Fire Data ptrs (5000 more lines) */
	FireDataAllocate(fire_data);

	/* Read all the input data */
	printf("Reading input...\n");
	ReadInputs(in_info, jugg);
	
	/* Mark day/night, glint, missing, etc. */
	printf("Setting up...\n");
	Setup(in_info, jugg, fire_data);
	
	/* Free input QFs */
	free_qf(jugg);

	printf("Day pixels: %d -- Night pixels: %d\n", jugg->day_count, jugg->night_count);

	/* Detect, classify and count day pixels */
	if (jugg->day_count > 0){
		printf("Detect Daytime...\n");
		Detect_Daytime(jugg, fire_data);
	
		printf("Postscreen Daytime...\n");
		Postscreen_Daytime(jugg, fire_data);
	}


	/* Detect, classify and count night pixels */
	if (jugg->night_count > 0){
		printf("Detect Nighttime...\n");
		Detect_Nighttime(jugg, fire_data);

		if (jugg->sa_night_pass == 1){
			printf("Create T13...\n");
			Create_T13(in_info, jugg);
			
			printf("Postscreen Nighttime...\n");
			Postscreen_Nighttime(jugg, fire_data);
		}
	}

	/* Replace water pixels */
	ReplaceWater(jugg);

	/* Holds points from ref file */
	StaticFireReference *txt_fire_ref;
	if ( !(txt_fire_ref = (StaticFireReference *) malloc(sizeof(StaticFireReference))) )
		return error(ERR_F_MEM);
	txt_fire_ref->entries = 0;

	/* Stored new points for ref file*/
	StaticFireReference *mem_fire_ref;
	if ( !(mem_fire_ref = (StaticFireReference *) malloc(sizeof(StaticFireReference))) )
		return error(ERR_F_MEM);
	mem_fire_ref->entries = 0;

	if(jugg->sa_night_pass == 0){
		printf("Create T13...\n");
		Create_T13(in_info, jugg);
	}
	
	/* Run lw_filter.c */
	LW_Filter(jugg, in_info, txt_fire_ref, mem_fire_ref);

	/* Replace folded channel 4 BT values with 367K */
	T4Folded(jugg);
	
	/* Claculate FRP */
	printf("Calculating FRP...\n");
	Calculate_FRP(jugg, fire_data);
	
	/* Calls the write functions needed (fire mask, fire data, ascii, global attr) */
	printf("Writing Output...\n");
	WriteOutput(out_info, in_info, jugg, fire_data, txt_fire_ref, mem_fire_ref);

	printf("Fires detected: %d\n", fire_data->index);

	/* Free memory */
	free_jugg(jugg);
	
	return 0;
}

/* Sets all the fire data ptrs to NULL */
/* Also sets the index & allocation varibles to 0 */
int FireDataInit(FireData *fire_data){
	fire_data->index = 0;
	fire_data->allocations = 0;
	fire_data->Mask  = NULL;
	fire_data->Lon = NULL;
	fire_data->Lat = NULL;
	fire_data->Row = NULL;
	fire_data->Col = NULL;
	fire_data->T4 = NULL;
	fire_data->T5 = NULL;
	fire_data->Window_Size = NULL;
	fire_data->SolZenAng = NULL;
	fire_data->SolAzAng = NULL;
	fire_data->ViewZenAng = NULL;
	fire_data->ViewAzAng = NULL;
	fire_data->Day = NULL;
	fire_data->mean_T4 = NULL;
	fire_data->mean_T5 = NULL;
	fire_data->MAD_T4 = NULL;
	fire_data->MAD_T5 = NULL;
	fire_data->mean_Dif45 = NULL;
	fire_data->MAD_Dif45 = NULL;
	fire_data->Adj_Cloud = NULL;
	fire_data->Adj_Water = NULL;
	fire_data->FRP = NULL;
	fire_data->Rad13 = NULL;
	fire_data->mean_Rad13 = NULL;


}

/* Realloc 5000 elements to FireData */
/* Called at the start and each time 5000 more fires are found */
int FireDataAllocate(FireData *fire_data){
	fire_data->allocations++;

	int allocs = fire_data->allocations;
	fire_data->Mask = (uint8*)realloc(fire_data->Mask, allocs*5000*sizeof(uint8));
	fire_data->Lon = (float*)realloc(fire_data->Lon, allocs*5000*sizeof(float));
	fire_data->Lat = (float*)realloc(fire_data->Lat, allocs*5000*sizeof(float));
	fire_data->Row = (uint16*)realloc(fire_data->Row, allocs*5000*sizeof(uint16));
	fire_data->Col = (uint16*)realloc(fire_data->Col, allocs*5000*sizeof(uint16));
	fire_data->T4 = (float*)realloc(fire_data->T4, allocs*5000*sizeof(float));
	fire_data->T5 = (float*)realloc(fire_data->T5, allocs*5000*sizeof(float));
	fire_data->Window_Size = (uint16*)realloc(fire_data->Window_Size, allocs*5000*sizeof(uint16));
	fire_data->SolZenAng = (float*)realloc(fire_data->SolZenAng, allocs*5000*sizeof(float));
	fire_data->SolAzAng = (float*)realloc(fire_data->SolAzAng, allocs*5000*sizeof(float));
	fire_data->ViewZenAng = (float*)realloc(fire_data->ViewZenAng, allocs*5000*sizeof(float));
	fire_data->ViewAzAng = (float*)realloc(fire_data->ViewAzAng, allocs*5000*sizeof(float));
	fire_data->Day = (uint8*)realloc(fire_data->Day, allocs*5000*sizeof(uint8));
	fire_data->mean_T4 = (float*)realloc(fire_data->mean_T4, allocs*5000*sizeof(float));
	fire_data->mean_T5 = (float*)realloc(fire_data->mean_T5, allocs*5000*sizeof(float));
	fire_data->MAD_T4 = (float*)realloc(fire_data->MAD_T4, allocs*5000*sizeof(float));
	fire_data->MAD_T5 = (float*)realloc(fire_data->MAD_T5, allocs*5000*sizeof(float));
	fire_data->mean_Dif45 = (float*)realloc(fire_data->mean_Dif45, allocs*5000*sizeof(float));
	fire_data->MAD_Dif45 = (float*)realloc(fire_data->MAD_Dif45, allocs*5000*sizeof(float));
	fire_data->Adj_Cloud = (uint16*)realloc(fire_data->Adj_Cloud, allocs*5000*sizeof(uint16));
	fire_data->Adj_Water = (uint16*)realloc(fire_data->Adj_Water, allocs*5000*sizeof(uint16));
	fire_data->FRP = (float*)realloc(fire_data->FRP, allocs*5000*sizeof(float));
	fire_data->Rad13 = (float*)realloc(fire_data->Rad13, allocs*5000*sizeof(float));
	fire_data->mean_Rad13 = (float*)realloc(fire_data->mean_Rad13, allocs*5000*sizeof(float));

}

/* Marks all pixels - day/night, missing, glint, etc. */
int Setup(InputInfo *in_info, Juggernaut *jugg, FireData *fire_data){
	int i, j, st, i_inc, j_inc;
	int day;
	int bad_geo;
	int missing_data;
	
	int glint;
	float GlintAng;	

	float T4, T5, Dif45;
	int R1, R2, R3, Ref12;
	int bad1, bad2, bad3, bad4, bad5;
	float RelAzAng;

	/* Alloc pixflags */
	if ( !(jugg->pixflags = (PixInfo**)malloc(jugg->number_of_scan_lines*sizeof(PixInfo*))) )
		return error(ERR_F_MEM);
	if ( !(jugg->pixflags[0] = (PixInfo*)malloc(jugg->number_of_scan_lines*SAMPLES_375M*sizeof(PixInfo))) )
		return error(ERR_F_MEM);
	for (i=1;i<(jugg->number_of_scan_lines);i++){
		jugg->pixflags[i] = jugg->pixflags[i-1] + SAMPLES_375M;
	}


	/* Runs over every pixel */
	for (i = 0; i < jugg->number_of_scan_lines; i++) {
		for (j = 0; j < SAMPLES_375M; j++) {
			jugg->pixflags[i][j].mask = 5;
			jugg->pixflags[i][j].meta_data = 0;

			/* clear loop flags*/
			bad1 = bad2 = bad3 = bad4 = bad5 = 0;

			if(jugg->QFGeo[i][j] != 0) jugg->pixflags[i][j].meta_data |= 0x20;	

			/* Meta data 6 */
			if(jugg->QF13[i/2][j/2] != 0){
				if(i%2 == 0){
					i_inc = 1;
				}else{
					i_inc = -1;
				}
				if(j%2 == 0){
					j_inc = 1;
				}else{
					j_inc = -1;
				}
				jugg->pixflags[i][j].meta_data |= 0x40;
				jugg->pixflags[i+i_inc][j].meta_data |= 0x40;
				jugg->pixflags[i][j+j_inc].meta_data |= 0x40;
				jugg->pixflags[i+i_inc][j+j_inc].meta_data |= 0x40;
			}

			/* Check day */
			if (jugg->SolZenAng[i][j] < 90 && jugg->SolZenAng[i][j] >= 0 && jugg->QFGeo[i][j] == 0){
				day = jugg->pixflags[i][j].day = 1;
			}else if(jugg->SolZenAng[i][j] >= 90 && jugg->QFGeo[i][j] == 0){
				day = jugg->pixflags[i][j].day = 0;
			}else{
				jugg->pixflags[i][j].mask = 0;
				jugg->missing_count += 1;
				continue;
			}

			/* Night pixels require different tests */
			if(!day){
				NighttimeSetup(jugg, fire_data, i, j);
				jugg->night_count += 1;
				continue;
			}
			jugg->day_count += 1;
		

			
			/* Get data */
			R1 = jugg->R1[i][j];
			R2 = jugg->R2[i][j];
			R3 = jugg->R3[i][j];

			T4 = jugg->T4[i][j];
			T5 = jugg->T5[i][j];

			Ref12 = R1 + R2;
			Dif45 = T4 - T5;

			/* sun-glint test */
			RelAzAng = fixang(jugg->SolAzAng[i][j] -
				jugg->ViewAzAng[i][j]);

			glint = SunGlint(Ref12, Dif45,
				jugg->SolZenAng[i][j],
				jugg->ViewZenAng[i][j],
				RelAzAng, &GlintAng);


			jugg->pixflags[i][j].glint_level = glint;
			if(glint != 0){
				jugg->glint_count += 1;
			}

			/* Test if cloud */
			if (iscloud(T5, Ref12)) {
				jugg->pixflags[i][j].mask = 4;
			}
			

			/* Test QF values */
			if(jugg->QF1[i][j] != 0) bad1 = 1;
			if(jugg->QF2[i][j] != 0) bad2 = 1;
			if(jugg->QF3[i][j] != 0) bad3 = 1;
			if(jugg->QF4[i][j] != 0) bad4 = 1;
			if(jugg->QF5[i][j] != 0) bad5 = 1;

			/* Meta Data 0-4 */
			if(bad1) jugg->pixflags[i][j].meta_data |= 0x01;
			if(bad2) jugg->pixflags[i][j].meta_data |= 0x02; 
			if(bad3) jugg->pixflags[i][j].meta_data |= 0x04; 
			if(bad4) jugg->pixflags[i][j].meta_data |= 0x08;
			if(bad5) jugg->pixflags[i][j].meta_data |= 0x10; 
			 

			/* Missing data test */
			missing_data = bad1 || bad2 || bad3 || bad4 || bad5;
			if (missing_data) {
				if(jugg->QF1[i][j] != 2 && jugg->QF2[i][j] != 2 && jugg->QF3[i][j] != 2 && jugg->QF4[i][j] != 2 && jugg->QF5[i][j] != 2){
					jugg->pixflags[i][j].missing_data = 1;
				}
				jugg->pixflags[i][j].mask = 0;
			}

			if(jugg->QF1[i][j] == 2 && jugg->QF2[i][j] == 2 && jugg->QF3[i][j] == 2 && jugg->QF4[i][j] == 2 && jugg->QF5[i][j] == 2){
				jugg->pixflags[i][j].mask = 1;
			}	


			/* Test I04 pixel saturation */
			if(jugg->QF4[i][j] == 9 && T4 == 367 && T5 > 290 && jugg->QF5[i][j] == 0 && Ref12 < 7000){
				jugg->pixflags[i][j].mask = 9;
			}


			/* Checking for water */
			if(R1 > R2 && R2 > R3){
				jugg->pixflags[i][j].water_mask = 1;
				jugg->water_count += 1;
			}else{
				jugg->pixflags[i][j].water_mask = 0;
				jugg->land_count += 1;
			}


			/* Test folding of I04 pixel */
			if(Dif45 < 0 && T5 > 325 && jugg->QF5[i][j] == 0){
				jugg->pixflags[i][j].mask = 9;
			}

			/* Test if background fire */
			if((T4 > 335 && Dif45 > 30) || jugg->pixflags[i][j].mask == 9){
				jugg->pixflags[i][j].bf_mask = 1;
				jugg->pixflags[i][j].meta_data |= 0x100;
			}

			/* Counters */
			if(jugg->pixflags[i][j].mask == 4){
				jugg->cloud_count += 1;
			}

			/* Missing count */
			if(jugg->pixflags[i][j].mask == 0){
				jugg->missing_count += 1;
			}

			/* Save fire data */
			if(jugg->pixflags[i][j].mask == 9){
				/* Fill in the necessary fire data */
				fire_data->Row[fire_data->index] = i;
				fire_data->Col[fire_data->index] = j;
				fire_data->mean_T4[fire_data->index] = 0;
				fire_data->mean_T5[fire_data->index] = 0;
				fire_data->MAD_T4[fire_data->index] = 0;
				fire_data->MAD_T5[fire_data->index] = 0;
				fire_data->mean_Dif45[fire_data->index] = 0;
				fire_data->MAD_Dif45[fire_data->index] = 0;
				fire_data->Window_Size[fire_data->index] = 0;
				fire_data->index++;
				/* Fill in the necessary fire data */
				if(fire_data->index%5000 == 0) FireDataAllocate(fire_data);


			}
		}
	}
}


/* Check nighttime pixel */
int NighttimeSetup(Juggernaut *jugg, FireData *fire_data, int i, int j){

	int missing_data;

	float T4, T5;
	float Dif45;
	int bad4 = 0;
	int bad5 = 0;

	if(jugg->lat[i][j] < 7 && jugg->lat[i][j] > -50 && jugg->lon[i][j] < 11 && jugg->lon[i][j] > -110) jugg->sa_night_pass = 1;

	/* Mark pixel as night */
	jugg->pixflags[i][j].night = 1;

	/* Get data */
	T4 = jugg->T4[i][j];
	T5 = jugg->T5[i][j];
	Dif45 = T4 - T5;

	/* Mask Clouds */
	if(T5 < 265 && T4 < 295){
		jugg->pixflags[i][j].mask = 4;
	}

	/* Test QF values */ 
	if(jugg->QF4[i][j] != 0) bad4 = 1;
	if(jugg->QF5[i][j] != 0) bad5 = 1;

	/* Meta Data 0-4 */
	if(bad4) jugg->pixflags[i][j].meta_data |= 0x08;
	if(bad5) jugg->pixflags[i][j].meta_data |= 0x10; 

	missing_data = bad4 || bad5;

	if (missing_data) {
		if(jugg->QF4[i][j] != 2 && jugg->QF5[i][j] != 2){
			jugg->pixflags[i][j].missing_data = 1;
		}
		jugg->pixflags[i][j].mask = 0;
	}
	if(jugg->QF4[i][j] == 2 && jugg->QF5[i][j] == 2) jugg->pixflags[i][j].mask = 1;

	/* Flag unambiguous fire pixels */
	if(T4 > 320 && bad4 == 0){
		jugg->pixflags[i][j].mask = 8;
		/* Meta data 7 */
		jugg->pixflags[i][j].meta_data |= 0x80;
	}

	/* Identify I04 pixel saturation */
	if(jugg->QF4[i][j] == 9 && T4 == 367 && bad5 == 0){
		jugg->pixflags[i][j].mask = 9;
	}

	/* Flag pixel folding */
	if(Dif45 < 0 && T5 > 310 && bad5 == 0){
		jugg->pixflags[i][j].mask = 9;
	}
	if(T4 == 208 && T5 > 335){
		jugg->pixflags[i][j].mask = 9;
	}


	/* Mask background fires */
	if( (T4 > 300 && Dif45 > 10) || (jugg->pixflags[i][j].mask == 9) ){
		/* Meta data 9 */
		jugg->pixflags[i][j].meta_data |= 0x100;
		jugg->pixflags[i][j].bf_mask = 1;
	}

	/* Counters */
	if(jugg->pixflags[i][j].mask == 4){
		jugg->cloud_count += 1;
	}

	if(jugg->pixflags[i][j].mask == 0){
		jugg->missing_count += 1;
	}

	/* Save fire data */
	if(jugg->pixflags[i][j].mask == 8 || jugg->pixflags[i][j].mask == 9){
		/* Fill in the necessary fire data */
		fire_data->Row[fire_data->index] = i;
		fire_data->Col[fire_data->index] = j;
		fire_data->mean_T4[fire_data->index] = 0;
		fire_data->mean_T5[fire_data->index] = 0;
		fire_data->MAD_T4[fire_data->index] = 0;
		fire_data->MAD_T5[fire_data->index] = 0;
		fire_data->mean_Dif45[fire_data->index] = 0;
		fire_data->MAD_Dif45[fire_data->index] = 0;
		fire_data->Window_Size[fire_data->index] = 0;
		fire_data->index++;
		/* Fill in the necessary fire data */
		if(fire_data->index%5000 == 0) FireDataAllocate(fire_data);


	}
}


/* Create T13 from aggregated and unaggregated M13 Radiance */
int Create_T13(InputInfo *in_info, Juggernaut *jugg){
	float **Rad13_unagg;
	int i, j, st, sds_id;
	float C1 = 119106600;
	float C2 = 14388.33;
	float lambda_13 = 4.07;
	int x,xx;
	int m_scan_lines = jugg->number_of_scan_lines / 2;
	int start[2], edges[2];

	start[0] = 0;
	edges[0] = m_scan_lines;

	start[1] = 0;
	edges[1] = SAMPLES_750M;

	/* DRL: In the DRL implementation, it's possible for rad13_unagg to have index of 0 since we create our own VCDGIPS HDF4 file
	if(in_info->sds_index_rad13_unagg == NULL){
		fputs("Error: Nighttime pass over South America encountered, but unaggregated input was not supplied.\n", stderr);
		return 1;
	}
	*/

	/* ----- Read M13 (unagg) ----- */
	start[1] = 0;
	edges[1] = SAMPLES_UNAGG;

	/* M13 (unagg) radiance tempature  */
	if ( !(Rad13_unagg = (float**)malloc(m_scan_lines*sizeof(float*))) )
		return error(ERR_F_MEM);
	if ( !(Rad13_unagg[0] = (float*)malloc(m_scan_lines*SAMPLES_UNAGG*sizeof(float))) )
		return error(ERR_F_MEM);
	for (i=1;i<m_scan_lines;i++){
		Rad13_unagg[i] = Rad13_unagg[i-1] + SAMPLES_UNAGG;
	}

	if ((sds_id = SDselect(in_info->sd_id_vcdgip, in_info->sds_index_rad13_unagg)) == FAIL) return ERR_HDF;
	if (SDreaddata(sds_id, start, NULL, edges, Rad13_unagg[0]) == FAIL)
		return ERR_HDF;
	/* ------ ------- */


	/* Init T13 */
	if ( !(jugg->T13 = (float**)malloc(m_scan_lines*sizeof(float*))) )
		return error(ERR_F_MEM);
	if ( !(jugg->T13[0] = (float*)malloc(m_scan_lines*SAMPLES_750M*sizeof(float))) )
		return error(ERR_F_MEM);
	for (i=1;i<m_scan_lines;i++){
		jugg->T13[i] = jugg->T13[i-1] + SAMPLES_750M;
	}

	int tempx;
	/* 1:1 Left side  */
	for(i=0;i<m_scan_lines;i++){ 
		for(j=0;j<640;j++){
			if(jugg->Rad13_agg[i][j] < 0){
				jugg->T13[i][j] = -1;
			}else{
				jugg->T13[i][j] = (C2/lambda_13) / (log(1+ C1/(jugg->Rad13_agg[i][j]*pow(lambda_13,5))));
			}
		}
	} 
	
	/* 1:1 Right side  */
	for(i=0;i<m_scan_lines;i++){ 
		for(j=2560;j<SAMPLES_750M;j++){	
			if(jugg->Rad13_agg[i][j] < 0){
				jugg->T13[i][j] = -1;
			}else{
				jugg->T13[i][j] = (C2/lambda_13) / (log(1+ C1/(jugg->Rad13_agg[i][j]*pow(lambda_13,5))));
			}
			
		}
	}

	/* 2:1 & 3:1  */
	for(i=0;i<m_scan_lines;i++){
		xx = 640;
		x  = 0;
		while(x < 5023){
			if(x <= 734 || x >= 4287){
				if(MAX(Rad13_unagg[i][x], Rad13_unagg[i][x+1]) < 0){
					jugg->T13[i][xx] = -1;
				}else{
					jugg->T13[i][xx] = (C2/lambda_13) / log(1+C1/(MAX(Rad13_unagg[i][x], Rad13_unagg[i][x+1])*pow(lambda_13,5)) );
				}
				x=x+2;
			}else{
				if(MAX(MAX(Rad13_unagg[i][x], Rad13_unagg[i][x+1]),Rad13_unagg[i][x+2]) < 0){
					jugg->T13[i][xx] = -1;
				}else{
					jugg->T13[i][xx] = (C2/lambda_13) / log(1+C1/(MAX(MAX(Rad13_unagg[i][x], Rad13_unagg[i][x+1]),Rad13_unagg[i][x+2])*pow(lambda_13,5)) );
				}
				x=x+3;
			}
			xx++;
		}	
	}

	free(Rad13_unagg[0]);
	free(Rad13_unagg);

}





static int SunGlint(float ref12, float dif45, float ts, float tv,
	float phi, float *GlintAng)
/*
!C*****************************************************************************
!Description:

!Input Parameters:
	ref12 is ref1 + ref2
	dif45 is bt4 - bt5
	ts	solar zenith angle ("theta sprintf("post lw\n");") in degrees
	tv	view zenith angle ("theta v") in degrees
	phi	relative azimuth angle in degrees

!Output Parameters:
	GlintAng	"glint" angle (degrees)

!Input/Output Parameters:
	None.

!Returns:
	0 if sun glint absent, otherwise glint level from 1 (lowest)
	to 2 (highest)

!Revision History
	Adapted from MODIS Collection 6 fire code, L. Giglio, 2012.

!END***************************************************************************
*/
{
	float cos_tr;		/* cosine of reflected sun angle "theta r" */
	float tr;

	cos_tr = cosf(D2RF * tv) * cosf(D2RF * ts) -
		sinf(D2RF * tv) * sinf(D2RF * ts) * cosf(D2RF * phi);

	tr = R2D * acosf(cos_tr);

	*GlintAng = tr;

	if (tr < 15.0F) return 2;

	if (tr < 25.0F) return 1;

	return 0;
}




static float fixang(float ang)
/*
!C*****************************************************************************
!Description:
	Map angle in range [-540,+540] to range [-180,+180].

!Input Parameters:
	ang	angle (degrees)

!Output Parameters:
	None.

!Input/Output Parameters:
	None.

!Returns:
	Returns angle mapped to range [-180,+180]

!Revision History

!References and Credits

!Design Notes

!END***************************************************************************
*/
{
	if (ang > +180.0F) return ang - 360.0F;
	if (ang <= -180.0F) return 360.0F + ang;
	return ang;
}

/* Returns 0 if clear pixel, non-zero if cloudy pixel */
int iscloud(float T5, int Ref12){
	int test1, test2a, test2b, test3a, test3b;

	test1 = T5 < 265;

	test2a = Ref12 > 9000;
	test2b = T5 < 295;

	test3a = Ref12 > 7000;
	test3b = T5 < 285.0F;

	return test1 || (test2a && test2b) || (test3a && test3b);
}

void T4Folded(Juggernaut *jugg){
	int i,j;
	for(i=0;i<jugg->number_of_scan_lines;i++){
		for(j=0;j<SAMPLES_375M;j++){
			if ( jugg->pixflags[i][j].mask == 9 || (jugg->pixflags[i][j].mask == 7 && jugg->T4[i][j] <= 350 && jugg->T5[i][j] >= 325) ){
				jugg->T4[i][j] = 367;
			}
		}
	}
}

void free_qf(Juggernaut *jugg){
	free(jugg->QF1[0]);
	free(jugg->QF1);
	free(jugg->QF2[0]);
	free(jugg->QF2);
	free(jugg->QF3[0]);
	free(jugg->QF3);
	free(jugg->QF4[0]);
	free(jugg->QF4);
	free(jugg->QF5[0]);
	free(jugg->QF5);
	free(jugg->QFGeo[0]);
	free(jugg->QFGeo);
}


void free_jugg(Juggernaut *jugg){
	/* Free memory */
	free(jugg->SolZenAng[0]);
	free(jugg->SolZenAng);
	free(jugg->SolAzAng[0]);
	free(jugg->SolAzAng);
	free(jugg->ViewZenAng[0]);
	free(jugg->ViewZenAng);
	free(jugg->ViewAzAng[0]);
	free(jugg->ViewAzAng);
	free(jugg->T4[0]);
	free(jugg->T4);
	free(jugg->T5[0]);
	free(jugg->T5);
	free(jugg->R1[0]);
	free(jugg->R1);
	free(jugg->R2[0]);
	free(jugg->R2);
	free(jugg->R3[0]);
	free(jugg->R3);
	free(jugg->lat[0]);
	free(jugg->lat);
	free(jugg->lon[0]);
	free(jugg->lon);
	free(jugg->MidTime);
	free(jugg->pixflags[0]);
	free(jugg->pixflags);
	free(jugg->Rad13_agg[0]);
	free(jugg->Rad13_agg);
	free(jugg->QF13[0]);
	free(jugg->QF13);
	free(jugg);
}

void free_firedata(FireData *fire_data){

	free(fire_data->index);
	free(fire_data->allocations);

	free(fire_data->Mask);
	free(fire_data->Lon);
	free(fire_data->Lat);
	free(fire_data->Row);
	free(fire_data->Col);
	free(fire_data->T4);
	free(fire_data->T5);
	free(fire_data->Window_Size);
	free(fire_data->SolZenAng);
	free(fire_data->SolAzAng);
	free(fire_data->ViewZenAng);
	free(fire_data->ViewAzAng);
	free(fire_data->Day);
	free(fire_data->mean_T4);
	free(fire_data->mean_T5);
	free(fire_data->MAD_T4);
	free(fire_data->MAD_T5);
	free(fire_data->mean_Dif45);
	free(fire_data->MAD_Dif45);
	free(fire_data->Adj_Cloud);
	free(fire_data->Adj_Water);
	free(fire_data->FRP);
	free(fire_data->Rad13);
	free(fire_data->mean_Rad13);

	free(fire_data);
}

