/*
!References and Credits:

	W. Schroeder, University of Maryland
	W. Walsh, University of Maryland
	
	Some of the framework was taken from M-band active fires, May 2012, L. Giglio.

	Last Edit: August 2016
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <unistd.h>
#include <time.h>
#include <sys/time.h>
#include <sys/resource.h>
#include "const.h"
#include "types.h"
#include "output.h"
#include "input.h"
#include "process.h"
#include "error.h"
#include "version.h"


int main(int argc, char **argv){
	int st, c;

	InputInfo in_info;
	OutputInfo out_info;

	struct rusage rusage;
	clock_t start = clock();

	extern char *optarg;
	extern int optind, opterr;

	/* Command line flags */
	in_info.verbose = out_info.write_ascii = out_info.write_hdf = out_info.write_netcdf = 0;

	/* Read command line arguments */
	while ((c = getopt(argc, argv, "a:m:dvn")) != -1) {
		switch (c) {
			case 'v':
				in_info.verbose = 1;
				break;
			case 'a':
				out_info.write_ascii = 1;
				out_info.ascii_name = optarg;
				break;
			case 'd':
				out_info.write_hdf = 1;
				break;
			case 'n':
				out_info.write_netcdf = 1;
				break;
			default:
				USAGE:
				fputs("Usage: vfire375 [-a ascii_filename -dv] viae_file imft_file vmae_file vcdgip_file qslwmip_file persistent_reference_file hdf_output\n",
					stderr);
				fputs("    a	Write fire pixel data to ASCII file.\n", stderr);
				fputs("    d	Append fire pixel SDSs (sparse arrays) to product.\n", stderr);
				fputs("    n	Write product as netCDF.\n", stderr);
				fputs("    v	Verbose mode.\n", stderr);
				fprintf(stderr, "\nVersion %s, compiled %s %s.\n",
					PROCESS_VERSION_NUMBER, __DATE__, __TIME__);
				exit(1);
				
				break;
		}
	}

	/* Make sure all input files are there */
	if (optind != argc - 7) goto USAGE;
	in_info.infile_viae = argv[optind];
	in_info.infile_imft = argv[optind+1];
	in_info.infile_vmae = argv[optind+2];
	in_info.infile_vcdgip = argv[optind+3];
	in_info.infile_qslwmip = argv[optind+4];
	in_info.infile_ref = argv[optind+5];
	out_info.hdf_name = argv[optind+6];
	
	/* Initalize the input -- Stores the sds indexs in in_info struct */
	if (st = InitInputHDF(&in_info)){
		printf("Error initializing the input\n");
		exit(1);
	}

	/* Initalize the output -- Stores the sds indexs in out_info struct, and adds fire_mask and algo qa */
	if (out_info.write_netcdf == 0){
		if (st = InitOutputHDF(&out_info, &in_info)){
			printf("Error initializing the output\n");
			exit(2);
		}
	}else{
		if (st = InitOutputNetCDF(&out_info, &in_info)){
			printf("Error initializing the output\n");
			exit(2);
		}
	}


	
	/* Main function */
	st = ProcessInput(&in_info, &out_info);
	if (st) {
		printf("Error processing the data\n");
		exit(3);
	}


	/* Print peak memory usage */
	getrusage( RUSAGE_SELF, &rusage );
	if(in_info.verbose) printf("Peak memory: %fGB\n", rusage.ru_maxrss/pow(2,20));

	/* Print time elapsed */
	clock_t end = clock();
	double diff = (double)(end - start) / CLOCKS_PER_SEC;
	if(in_info.verbose) printf("Time elapsed: %lf seconds\n", diff);

	return 0;
}

